"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from enum import Enum
from r9s import errors, models, utils
from r9s._hooks import HookContext
from r9s.types import OptionalNullable, UNSET
from r9s.utils import eventstreaming
from r9s.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Literal, Mapping, Optional, Union, overload


class CreateAcceptEnum(str, Enum):
    APPLICATION_JSON = "application/json"
    TEXT_EVENT_STREAM = "text/event-stream"


class Chat(BaseSDK):
    @overload
    def create(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        frequency_penalty: Optional[float] = None,
        logit_bias: Optional[Dict[str, float]] = None,
        logprobs: Optional[bool] = None,
        top_logprobs: Optional[int] = None,
        max_tokens: Optional[int] = None,
        n: Optional[int] = None,
        modalities: Optional[List[models.Modalities]] = None,
        audio: Optional[Union[models.Audio, models.AudioTypedDict]] = None,
        presence_penalty: Optional[float] = None,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: Optional[int] = None,
        service_tier: Optional[models.ServiceTier] = None,
        stop: Optional[Union[models.Stop, models.StopTypedDict]] = None,
        stream: Union[Literal[False], None] = None,
        stream_options: Optional[
            Union[models.StreamOptions, models.StreamOptionsTypedDict]
        ] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[Union[List[models.Tool], List[models.ToolTypedDict]]] = None,
        tool_choice: Optional[
            Union[
                models.ChatCompletionRequestToolChoice,
                models.ChatCompletionRequestToolChoiceTypedDict,
            ]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        user: Optional[str] = None,
        reasoning_effort: Optional[models.ReasoningEffort] = None,
        max_completion_tokens: Optional[int] = None,
        store: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ChatCompletionResponse:
        r"""Create chat completion

        Create a chat completion, supports streaming

        :param model: Model name
        :param messages: Messages list
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs: When true, stream must be false (OpenAI constraint)
        :param top_logprobs:
        :param max_tokens:
        :param n: Number of chat completion choices to generate
        :param modalities: Output modality types. Use [\"text\", \"audio\"] for audio output
        :param audio:
        :param presence_penalty:
        :param response_format:
        :param seed:
        :param service_tier:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter (non-OpenAI standard, model-specific)
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use. Only valid when tools are specified.
        :param user: Unique identifier representing end-user for abuse monitoring
        :param reasoning_effort: Reasoning effort level for o1 series models (low, medium, high)
        :param max_completion_tokens: Maximum number of tokens to generate in the completion (alternative to max_tokens, more precise)
        :param store: Whether to store the output for use in model distillation or evals
        :param metadata: Custom metadata to attach to the request for tracking purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    def create(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        frequency_penalty: Optional[float] = None,
        logit_bias: Optional[Dict[str, float]] = None,
        logprobs: Optional[bool] = None,
        top_logprobs: Optional[int] = None,
        max_tokens: Optional[int] = None,
        n: Optional[int] = None,
        modalities: Optional[List[models.Modalities]] = None,
        audio: Optional[Union[models.Audio, models.AudioTypedDict]] = None,
        presence_penalty: Optional[float] = None,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: Optional[int] = None,
        service_tier: Optional[models.ServiceTier] = None,
        stop: Optional[Union[models.Stop, models.StopTypedDict]] = None,
        stream: Literal[True],
        stream_options: Optional[
            Union[models.StreamOptions, models.StreamOptionsTypedDict]
        ] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[Union[List[models.Tool], List[models.ToolTypedDict]]] = None,
        tool_choice: Optional[
            Union[
                models.ChatCompletionRequestToolChoice,
                models.ChatCompletionRequestToolChoiceTypedDict,
            ]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        user: Optional[str] = None,
        reasoning_effort: Optional[models.ReasoningEffort] = None,
        max_completion_tokens: Optional[int] = None,
        store: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[models.ChatCompletionStreamEvent]:
        r"""Create chat completion

        Create a chat completion, supports streaming

        :param model: Model name
        :param messages: Messages list
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs: When true, stream must be false (OpenAI constraint)
        :param top_logprobs:
        :param max_tokens:
        :param n: Number of chat completion choices to generate
        :param modalities: Output modality types. Use [\"text\", \"audio\"] for audio output
        :param audio:
        :param presence_penalty:
        :param response_format:
        :param seed:
        :param service_tier:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter (non-OpenAI standard, model-specific)
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use. Only valid when tools are specified.
        :param user: Unique identifier representing end-user for abuse monitoring
        :param reasoning_effort: Reasoning effort level for o1 series models (low, medium, high)
        :param max_completion_tokens: Maximum number of tokens to generate in the completion (alternative to max_tokens, more precise)
        :param store: Whether to store the output for use in model distillation or evals
        :param metadata: Custom metadata to attach to the request for tracking purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    def create(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        frequency_penalty: Optional[float] = None,
        logit_bias: Optional[Dict[str, float]] = None,
        logprobs: Optional[bool] = None,
        top_logprobs: Optional[int] = None,
        max_tokens: Optional[int] = None,
        n: Optional[int] = None,
        modalities: Optional[List[models.Modalities]] = None,
        audio: Optional[Union[models.Audio, models.AudioTypedDict]] = None,
        presence_penalty: Optional[float] = None,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: Optional[int] = None,
        service_tier: Optional[models.ServiceTier] = None,
        stop: Optional[Union[models.Stop, models.StopTypedDict]] = None,
        stream: Optional[bool] = False,
        stream_options: Optional[
            Union[models.StreamOptions, models.StreamOptionsTypedDict]
        ] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[Union[List[models.Tool], List[models.ToolTypedDict]]] = None,
        tool_choice: Optional[
            Union[
                models.ChatCompletionRequestToolChoice,
                models.ChatCompletionRequestToolChoiceTypedDict,
            ]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        user: Optional[str] = None,
        reasoning_effort: Optional[models.ReasoningEffort] = None,
        max_completion_tokens: Optional[int] = None,
        store: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateChatCompletionResponse:
        r"""Create chat completion

        Create a chat completion, supports streaming

        :param model: Model name
        :param messages: Messages list
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs: When true, stream must be false (OpenAI constraint)
        :param top_logprobs:
        :param max_tokens:
        :param n: Number of chat completion choices to generate
        :param modalities: Output modality types. Use [\"text\", \"audio\"] for audio output
        :param audio:
        :param presence_penalty:
        :param response_format:
        :param seed:
        :param service_tier:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter (non-OpenAI standard, model-specific)
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use. Only valid when tools are specified.
        :param user: Unique identifier representing end-user for abuse monitoring
        :param reasoning_effort: Reasoning effort level for o1 series models (low, medium, high)
        :param max_completion_tokens: Maximum number of tokens to generate in the completion (alternative to max_tokens, more precise)
        :param store: Whether to store the output for use in model distillation or evals
        :param metadata: Custom metadata to attach to the request for tracking purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatCompletionRequest(
            model=model,
            messages=utils.get_pydantic_model(messages, List[models.Message]),
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            top_logprobs=top_logprobs,
            max_tokens=max_tokens,
            n=n,
            modalities=modalities,
            audio=utils.get_pydantic_model(audio, Optional[models.Audio]),
            presence_penalty=presence_penalty,
            response_format=utils.get_pydantic_model(
                response_format, Optional[models.ResponseFormat]
            ),
            seed=seed,
            service_tier=service_tier,
            stop=stop,
            stream=stream,
            stream_options=utils.get_pydantic_model(
                stream_options, Optional[models.StreamOptions]
            ),
            temperature=temperature,
            top_p=top_p,
            top_k=top_k,
            tools=utils.get_pydantic_model(tools, Optional[List[models.Tool]]),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[models.ChatCompletionRequestToolChoice]
            ),
            parallel_tool_calls=parallel_tool_calls,
            user=user,
            reasoning_effort=reasoning_effort,
            max_completion_tokens=max_completion_tokens,
            store=store,
            metadata=metadata,
        )

        req = self._build_request(
            method="POST",
            path="/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatCompletionRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createChatCompletion",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return unmarshal_json_response(
                models.ChatCompletionResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, models.CreateChatCompletionResponseBody
                ).data,
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res, http_res_text
            )
            raise errors.BadRequestError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.AuthenticationErrorData, http_res, http_res_text
            )
            raise errors.AuthenticationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "403", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.PermissionDeniedErrorData, http_res, http_res_text
            )
            raise errors.PermissionDeniedError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.RateLimitErrorData, http_res, http_res_text
            )
            raise errors.RateLimitError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res, http_res_text
            )
            raise errors.InternalServerError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)

        http_res_text = utils.stream_to_text(http_res)
        raise errors.R9SDefaultError(
            "Unexpected response received", http_res, http_res_text
        )

    @overload
    async def create_async(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        frequency_penalty: Optional[float] = None,
        logit_bias: Optional[Dict[str, float]] = None,
        logprobs: Optional[bool] = None,
        top_logprobs: Optional[int] = None,
        max_tokens: Optional[int] = None,
        n: Optional[int] = None,
        modalities: Optional[List[models.Modalities]] = None,
        audio: Optional[Union[models.Audio, models.AudioTypedDict]] = None,
        presence_penalty: Optional[float] = None,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: Optional[int] = None,
        service_tier: Optional[models.ServiceTier] = None,
        stop: Optional[Union[models.Stop, models.StopTypedDict]] = None,
        stream: Union[Literal[False], None] = None,
        stream_options: Optional[
            Union[models.StreamOptions, models.StreamOptionsTypedDict]
        ] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[Union[List[models.Tool], List[models.ToolTypedDict]]] = None,
        tool_choice: Optional[
            Union[
                models.ChatCompletionRequestToolChoice,
                models.ChatCompletionRequestToolChoiceTypedDict,
            ]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        user: Optional[str] = None,
        reasoning_effort: Optional[models.ReasoningEffort] = None,
        max_completion_tokens: Optional[int] = None,
        store: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ChatCompletionResponse:
        r"""Create chat completion

        Create a chat completion, supports streaming

        :param model: Model name
        :param messages: Messages list
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs: When true, stream must be false (OpenAI constraint)
        :param top_logprobs:
        :param max_tokens:
        :param n: Number of chat completion choices to generate
        :param modalities: Output modality types. Use [\"text\", \"audio\"] for audio output
        :param audio:
        :param presence_penalty:
        :param response_format:
        :param seed:
        :param service_tier:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter (non-OpenAI standard, model-specific)
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use. Only valid when tools are specified.
        :param user: Unique identifier representing end-user for abuse monitoring
        :param reasoning_effort: Reasoning effort level for o1 series models (low, medium, high)
        :param max_completion_tokens: Maximum number of tokens to generate in the completion (alternative to max_tokens, more precise)
        :param store: Whether to store the output for use in model distillation or evals
        :param metadata: Custom metadata to attach to the request for tracking purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    async def create_async(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        frequency_penalty: Optional[float] = None,
        logit_bias: Optional[Dict[str, float]] = None,
        logprobs: Optional[bool] = None,
        top_logprobs: Optional[int] = None,
        max_tokens: Optional[int] = None,
        n: Optional[int] = None,
        modalities: Optional[List[models.Modalities]] = None,
        audio: Optional[Union[models.Audio, models.AudioTypedDict]] = None,
        presence_penalty: Optional[float] = None,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: Optional[int] = None,
        service_tier: Optional[models.ServiceTier] = None,
        stop: Optional[Union[models.Stop, models.StopTypedDict]] = None,
        stream: Literal[True],
        stream_options: Optional[
            Union[models.StreamOptions, models.StreamOptionsTypedDict]
        ] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[Union[List[models.Tool], List[models.ToolTypedDict]]] = None,
        tool_choice: Optional[
            Union[
                models.ChatCompletionRequestToolChoice,
                models.ChatCompletionRequestToolChoiceTypedDict,
            ]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        user: Optional[str] = None,
        reasoning_effort: Optional[models.ReasoningEffort] = None,
        max_completion_tokens: Optional[int] = None,
        store: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[models.ChatCompletionStreamEvent]:
        r"""Create chat completion

        Create a chat completion, supports streaming

        :param model: Model name
        :param messages: Messages list
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs: When true, stream must be false (OpenAI constraint)
        :param top_logprobs:
        :param max_tokens:
        :param n: Number of chat completion choices to generate
        :param modalities: Output modality types. Use [\"text\", \"audio\"] for audio output
        :param audio:
        :param presence_penalty:
        :param response_format:
        :param seed:
        :param service_tier:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter (non-OpenAI standard, model-specific)
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use. Only valid when tools are specified.
        :param user: Unique identifier representing end-user for abuse monitoring
        :param reasoning_effort: Reasoning effort level for o1 series models (low, medium, high)
        :param max_completion_tokens: Maximum number of tokens to generate in the completion (alternative to max_tokens, more precise)
        :param store: Whether to store the output for use in model distillation or evals
        :param metadata: Custom metadata to attach to the request for tracking purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    async def create_async(
        self,
        *,
        model: str,
        messages: Union[List[models.Message], List[models.MessageTypedDict]],
        frequency_penalty: Optional[float] = None,
        logit_bias: Optional[Dict[str, float]] = None,
        logprobs: Optional[bool] = None,
        top_logprobs: Optional[int] = None,
        max_tokens: Optional[int] = None,
        n: Optional[int] = None,
        modalities: Optional[List[models.Modalities]] = None,
        audio: Optional[Union[models.Audio, models.AudioTypedDict]] = None,
        presence_penalty: Optional[float] = None,
        response_format: Optional[
            Union[models.ResponseFormat, models.ResponseFormatTypedDict]
        ] = None,
        seed: Optional[int] = None,
        service_tier: Optional[models.ServiceTier] = None,
        stop: Optional[Union[models.Stop, models.StopTypedDict]] = None,
        stream: Optional[bool] = False,
        stream_options: Optional[
            Union[models.StreamOptions, models.StreamOptionsTypedDict]
        ] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[Union[List[models.Tool], List[models.ToolTypedDict]]] = None,
        tool_choice: Optional[
            Union[
                models.ChatCompletionRequestToolChoice,
                models.ChatCompletionRequestToolChoiceTypedDict,
            ]
        ] = None,
        parallel_tool_calls: Optional[bool] = None,
        user: Optional[str] = None,
        reasoning_effort: Optional[models.ReasoningEffort] = None,
        max_completion_tokens: Optional[int] = None,
        store: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateChatCompletionResponse:
        r"""Create chat completion

        Create a chat completion, supports streaming

        :param model: Model name
        :param messages: Messages list
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs: When true, stream must be false (OpenAI constraint)
        :param top_logprobs:
        :param max_tokens:
        :param n: Number of chat completion choices to generate
        :param modalities: Output modality types. Use [\"text\", \"audio\"] for audio output
        :param audio:
        :param presence_penalty:
        :param response_format:
        :param seed:
        :param service_tier:
        :param stop:
        :param stream:
        :param stream_options:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter (non-OpenAI standard, model-specific)
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls: Whether to enable parallel function calling during tool use. Only valid when tools are specified.
        :param user: Unique identifier representing end-user for abuse monitoring
        :param reasoning_effort: Reasoning effort level for o1 series models (low, medium, high)
        :param max_completion_tokens: Maximum number of tokens to generate in the completion (alternative to max_tokens, more precise)
        :param store: Whether to store the output for use in model distillation or evals
        :param metadata: Custom metadata to attach to the request for tracking purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatCompletionRequest(
            model=model,
            messages=utils.get_pydantic_model(messages, List[models.Message]),
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            top_logprobs=top_logprobs,
            max_tokens=max_tokens,
            n=n,
            modalities=modalities,
            audio=utils.get_pydantic_model(audio, Optional[models.Audio]),
            presence_penalty=presence_penalty,
            response_format=utils.get_pydantic_model(
                response_format, Optional[models.ResponseFormat]
            ),
            seed=seed,
            service_tier=service_tier,
            stop=stop,
            stream=stream,
            stream_options=utils.get_pydantic_model(
                stream_options, Optional[models.StreamOptions]
            ),
            temperature=temperature,
            top_p=top_p,
            top_k=top_k,
            tools=utils.get_pydantic_model(tools, Optional[List[models.Tool]]),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[models.ChatCompletionRequestToolChoice]
            ),
            parallel_tool_calls=parallel_tool_calls,
            user=user,
            reasoning_effort=reasoning_effort,
            max_completion_tokens=max_completion_tokens,
            store=store,
            metadata=metadata,
        )

        req = self._build_request_async(
            method="POST",
            path="/chat/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatCompletionRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createChatCompletion",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return unmarshal_json_response(
                models.ChatCompletionResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, models.CreateChatCompletionResponseBody
                ).data,
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res, http_res_text
            )
            raise errors.BadRequestError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.AuthenticationErrorData, http_res, http_res_text
            )
            raise errors.AuthenticationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "403", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.PermissionDeniedErrorData, http_res, http_res_text
            )
            raise errors.PermissionDeniedError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.RateLimitErrorData, http_res, http_res_text
            )
            raise errors.RateLimitError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res, http_res_text
            )
            raise errors.InternalServerError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.R9SDefaultError(
            "Unexpected response received", http_res, http_res_text
        )



from __future__ import annotations

import sys
import threading
import time
from typing import Optional


class LoadingSpinner:
    """Context manager for displaying a loading animation."""

    def __init__(self, message: str = "Loading") -> None:
        self.message = message
        self.running = False
        self.thread: Optional[threading.Thread] = None

    def _animate(self) -> None:
        dots = 0
        while self.running:
            sys.stdout.write(f"\r{self.message}{'.' * dots}   ")
            sys.stdout.flush()
            dots = (dots + 1) % 4
            time.sleep(0.5)
        sys.stdout.write("\r" + " " * (len(self.message) + 10) + "\r")
        sys.stdout.flush()

    def __enter__(self) -> "LoadingSpinner":
        self.running = True
        self.thread = threading.Thread(target=self._animate, daemon=True)
        self.thread.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self.running = False
        if self.thread:
            self.thread.join(timeout=1)


class Spinner:
    def __init__(self, prefix: str) -> None:
        self._prefix = prefix
        self._stop = threading.Event()
        self._thread: Optional[threading.Thread] = None
        self._last_len = 0
        self.prefix_printed = False

    def start(self) -> None:
        self._thread = threading.Thread(target=self._run, daemon=True)
        self._thread.start()

    def print_prefix(self) -> None:
        if self.prefix_printed or not self._prefix:
            return
        sys.stdout.write(self._prefix)
        sys.stdout.flush()
        self.prefix_printed = True

    def stop_and_clear(self) -> None:
        if self._thread is None:
            return
        self._stop.set()
        self._thread.join(timeout=0.5)
        self._thread = None
        if self._last_len > 0:
            if self._prefix:
                sys.stdout.write(
                    "\r" + self._prefix + (" " * self._last_len) + "\r" + self._prefix
                )
            else:
                sys.stdout.write("\r" + (" " * self._last_len) + "\r")
            sys.stdout.flush()
        self._last_len = 0
        self.prefix_printed = True

    def _run(self) -> None:
        frames = ["|", "/", "-", "\\"]
        idx = 0
        while not self._stop.is_set():
            frame = frames[idx % len(frames)]
            anim = f" {frame}"
            self._last_len = len(anim)
            sys.stdout.write("\r" + (self._prefix or "") + anim)
            sys.stdout.flush()
            idx += 1
            time.sleep(0.12)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from pydantic import model_serializer
from r9s.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from typing import Any, List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ResponseOutputItemType = Literal[
    "message",
    "function_call",
    "reasoning",
]
r"""Output type (message for final response, function_call for tool calls, reasoning for reasoning trace)"""


ResponseOutputItemStatus = Literal[
    "completed",
    "in_progress",
    "incomplete",
]
r"""Output status"""


ResponseOutputItemRole = Literal[
    "user",
    "assistant",
]
r"""Message role"""


ResponseOutputItemSchemasType = Literal[
    "text",
    "refusal",
    "output_text",
    "reasoning_text",
]
r"""Content type (text for messages, output_text for responses, reasoning_text for reasoning traces, refusal for safety refusals)"""


class ResponseOutputItemContentTypedDict(TypedDict):
    type: NotRequired[ResponseOutputItemSchemasType]
    r"""Content type (text for messages, output_text for responses, reasoning_text for reasoning traces, refusal for safety refusals)"""
    text: NotRequired[str]
    r"""Text content"""
    refusal: NotRequired[str]
    r"""Refusal content"""
    annotations: NotRequired[List[Any]]
    r"""Content annotations"""
    logprobs: NotRequired[List[Any]]
    r"""Log probabilities"""


class ResponseOutputItemContent(BaseModel):
    type: Optional[ResponseOutputItemSchemasType] = None
    r"""Content type (text for messages, output_text for responses, reasoning_text for reasoning traces, refusal for safety refusals)"""

    text: Optional[str] = None
    r"""Text content"""

    refusal: Optional[str] = None
    r"""Refusal content"""

    annotations: Optional[List[Any]] = None
    r"""Content annotations"""

    logprobs: Optional[List[Any]] = None
    r"""Log probabilities"""


SummaryTypedDict = TypeAliasType("SummaryTypedDict", Union[str, List[Any]])
r"""Natural-language summary of reasoning (for reasoning type), can be string or array"""


Summary = TypeAliasType("Summary", Union[str, List[Any]])
r"""Natural-language summary of reasoning (for reasoning type), can be string or array"""


class ResponseOutputItemTypedDict(TypedDict):
    id: str
    r"""Unique identifier for output item"""
    type: ResponseOutputItemType
    r"""Output type (message for final response, function_call for tool calls, reasoning for reasoning trace)"""
    status: NotRequired[ResponseOutputItemStatus]
    r"""Output status"""
    role: NotRequired[ResponseOutputItemRole]
    r"""Message role"""
    content: NotRequired[List[ResponseOutputItemContentTypedDict]]
    r"""Content array"""
    call_id: NotRequired[str]
    r"""Function call ID"""
    name: NotRequired[str]
    r"""Function name"""
    arguments: NotRequired[str]
    r"""Function arguments (JSON string)"""
    output: NotRequired[str]
    r"""Function output"""
    summary: NotRequired[Nullable[SummaryTypedDict]]
    r"""Natural-language summary of reasoning (for reasoning type), can be string or array"""
    encrypted_content: NotRequired[Nullable[str]]
    r"""Encrypted reasoning tokens for stateless workflows (for reasoning type)"""


class ResponseOutputItem(BaseModel):
    id: str
    r"""Unique identifier for output item"""

    type: ResponseOutputItemType
    r"""Output type (message for final response, function_call for tool calls, reasoning for reasoning trace)"""

    status: Optional[ResponseOutputItemStatus] = None
    r"""Output status"""

    role: Optional[ResponseOutputItemRole] = None
    r"""Message role"""

    content: Optional[List[ResponseOutputItemContent]] = None
    r"""Content array"""

    call_id: Optional[str] = None
    r"""Function call ID"""

    name: Optional[str] = None
    r"""Function name"""

    arguments: Optional[str] = None
    r"""Function arguments (JSON string)"""

    output: Optional[str] = None
    r"""Function output"""

    summary: OptionalNullable[Summary] = UNSET
    r"""Natural-language summary of reasoning (for reasoning type), can be string or array"""

    encrypted_content: OptionalNullable[str] = UNSET
    r"""Encrypted reasoning tokens for stateless workflows (for reasoning type)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "status",
            "role",
            "content",
            "call_id",
            "name",
            "arguments",
            "output",
            "summary",
            "encrypted_content",
        ]
        nullable_fields = ["summary", "encrypted_content"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



from __future__ import annotations

from dataclasses import dataclass
from typing import Iterable, List

from r9s.cli_tools.ui.terminal import (
    BOLD,
    FG_ACCENT,
    FG_CMD,
    FG_MUTED,
    FG_NOTE,
    FG_TITLE,
    _style,
)


@dataclass(frozen=True)
class ExampleBlock:
    title: str
    notes: List[str]
    commands: List[str]


def _parse_example_block(raw: str) -> ExampleBlock:
    lines = [ln.rstrip() for ln in raw.splitlines()]
    title: str | None = None
    notes: List[str] = []
    commands: List[str] = []
    for ln in lines:
        s = ln.strip()
        if not s:
            continue
        if s.startswith("#"):
            text = s.lstrip("#").strip()
            if not text:
                continue
            if title is None:
                title = text
            else:
                notes.append(text)
        else:
            commands.append(s)
    return ExampleBlock(title=title or "Example", notes=notes, commands=commands)


def print_home(
    *,
    name: str,
    description: str,
    examples_title: str,
    examples: Iterable[str],
    footer: str,
) -> None:
    print(_style(name, BOLD, FG_TITLE))
    if description:
        print(_style(description, FG_MUTED))
    print()

    if examples_title:
        print(_style(examples_title, BOLD, FG_TITLE))

    for raw in examples:
        block = _parse_example_block(raw)
        if not block.commands:
            continue
        print(_style(f"- {block.title}", FG_ACCENT))
        for note in block.notes:
            print(_style(f"  {note}", FG_NOTE))
        for cmd in block.commands:
            print(_style(f"  {cmd}", FG_CMD))
        print()

    if footer:
        print(_style(footer, FG_MUTED))



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from pydantic.functional_validators import AfterValidator
from r9s.types import BaseModel
from r9s.utils import validate_const
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ToolChoiceFunctionTypedDict(TypedDict):
    name: str
    r"""Function name to call"""


class ToolChoiceFunction(BaseModel):
    name: str
    r"""Function name to call"""


class ToolChoiceTypedDict(TypedDict):
    r"""Specify specific tool selection"""

    function: ToolChoiceFunctionTypedDict
    type: Literal["function"]
    r"""Tool type"""


class ToolChoice(BaseModel):
    r"""Specify specific tool selection"""

    function: ToolChoiceFunction

    TYPE: Annotated[
        Annotated[Literal["function"], AfterValidator(validate_const("function"))],
        pydantic.Field(alias="type"),
    ] = "function"
    r"""Tool type"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from enum import Enum
from r9s import errors, models, utils
from r9s._hooks import HookContext
from r9s.types import OptionalNullable, UNSET
from r9s.utils import eventstreaming
from r9s.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Literal, Mapping, Optional, Union, overload


class CreateAcceptEnum(str, Enum):
    APPLICATION_JSON = "application/json"
    TEXT_EVENT_STREAM = "text/event-stream"


class Messages(BaseSDK):
    @overload
    def create(
        self,
        *,
        model: str,
        messages: Union[
            List[models.AnthropicMessageMessage],
            List[models.AnthropicMessageMessageTypedDict],
        ],
        system: Optional[str] = None,
        max_tokens: Optional[int] = None,
        stop_sequences: Optional[List[str]] = None,
        stream: Union[Literal[False], None] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[
            Union[List[models.AnthropicTool], List[models.AnthropicToolTypedDict]]
        ] = None,
        tool_choice: Optional[
            Union[
                models.AnthropicMessageRequestToolChoice,
                models.AnthropicMessageRequestToolChoiceTypedDict,
            ]
        ] = None,
        metadata: Optional[Dict[str, Any]] = None,
        thinking: Optional[Union[models.Thinking, models.ThinkingTypedDict]] = None,
        service_tier: Optional[models.AnthropicMessageRequestServiceTier] = "auto",
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AnthropicMessageResponse:
        r"""Create message (Claude native API)

        Create a message using Anthropic Claude's native API format, supports streaming

        :param model: Claude model name
        :param messages: Messages list, first message must be a user message
        :param system: System prompt
        :param max_tokens: Maximum number of output tokens (optional).
            If not provided, the relay service or API may use a default value.
            Different models have different maximum values.

        :param stop_sequences: Stop sequences
        :param stream:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter. Only sample from the top K options for each subsequent token.
        :param tools:
        :param tool_choice:
        :param metadata: An object describing metadata about the request. Can be used for tracking, identification, or filtering purposes.
            Common use cases: user_id, session_id, request_id, etc.

        :param thinking: Configuration for extended thinking (Claude 3.7+). When enabled, the model will spend more time thinking before responding.

        :param service_tier: Service tier for request processing:
            - auto: Automatically select between standard and priority capacity
            - standard_only: Only use standard capacity (may have longer wait times during high load)

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    def create(
        self,
        *,
        model: str,
        messages: Union[
            List[models.AnthropicMessageMessage],
            List[models.AnthropicMessageMessageTypedDict],
        ],
        system: Optional[str] = None,
        max_tokens: Optional[int] = None,
        stop_sequences: Optional[List[str]] = None,
        stream: Literal[True],
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[
            Union[List[models.AnthropicTool], List[models.AnthropicToolTypedDict]]
        ] = None,
        tool_choice: Optional[
            Union[
                models.AnthropicMessageRequestToolChoice,
                models.AnthropicMessageRequestToolChoiceTypedDict,
            ]
        ] = None,
        metadata: Optional[Dict[str, Any]] = None,
        thinking: Optional[Union[models.Thinking, models.ThinkingTypedDict]] = None,
        service_tier: Optional[models.AnthropicMessageRequestServiceTier] = "auto",
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[models.AnthropicStreamEvent]:
        r"""Create message (Claude native API)

        Create a message using Anthropic Claude's native API format, supports streaming

        :param model: Claude model name
        :param messages: Messages list, first message must be a user message
        :param system: System prompt
        :param max_tokens: Maximum number of output tokens (optional).
            If not provided, the relay service or API may use a default value.
            Different models have different maximum values.

        :param stop_sequences: Stop sequences
        :param stream:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter. Only sample from the top K options for each subsequent token.
        :param tools:
        :param tool_choice:
        :param metadata: An object describing metadata about the request. Can be used for tracking, identification, or filtering purposes.
            Common use cases: user_id, session_id, request_id, etc.

        :param thinking: Configuration for extended thinking (Claude 3.7+). When enabled, the model will spend more time thinking before responding.

        :param service_tier: Service tier for request processing:
            - auto: Automatically select between standard and priority capacity
            - standard_only: Only use standard capacity (may have longer wait times during high load)

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    def create(
        self,
        *,
        model: str,
        messages: Union[
            List[models.AnthropicMessageMessage],
            List[models.AnthropicMessageMessageTypedDict],
        ],
        system: Optional[str] = None,
        max_tokens: Optional[int] = None,
        stop_sequences: Optional[List[str]] = None,
        stream: Optional[bool] = False,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[
            Union[List[models.AnthropicTool], List[models.AnthropicToolTypedDict]]
        ] = None,
        tool_choice: Optional[
            Union[
                models.AnthropicMessageRequestToolChoice,
                models.AnthropicMessageRequestToolChoiceTypedDict,
            ]
        ] = None,
        metadata: Optional[Dict[str, Any]] = None,
        thinking: Optional[Union[models.Thinking, models.ThinkingTypedDict]] = None,
        service_tier: Optional[models.AnthropicMessageRequestServiceTier] = "auto",
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateMessageResponse:
        r"""Create message (Claude native API)

        Create a message using Anthropic Claude's native API format, supports streaming

        :param model: Claude model name
        :param messages: Messages list, first message must be a user message
        :param system: System prompt
        :param max_tokens: Maximum number of output tokens (optional).
            If not provided, the relay service or API may use a default value.
            Different models have different maximum values.

        :param stop_sequences: Stop sequences
        :param stream:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter. Only sample from the top K options for each subsequent token.
        :param tools:
        :param tool_choice:
        :param metadata: An object describing metadata about the request. Can be used for tracking, identification, or filtering purposes.
            Common use cases: user_id, session_id, request_id, etc.

        :param thinking: Configuration for extended thinking (Claude 3.7+). When enabled, the model will spend more time thinking before responding.

        :param service_tier: Service tier for request processing:
            - auto: Automatically select between standard and priority capacity
            - standard_only: Only use standard capacity (may have longer wait times during high load)

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AnthropicMessageRequest(
            model=model,
            messages=utils.get_pydantic_model(
                messages, List[models.AnthropicMessageMessage]
            ),
            system=system,
            max_tokens=max_tokens,
            stop_sequences=stop_sequences,
            stream=stream,
            temperature=temperature,
            top_p=top_p,
            top_k=top_k,
            tools=utils.get_pydantic_model(tools, Optional[List[models.AnthropicTool]]),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[models.AnthropicMessageRequestToolChoice]
            ),
            metadata=metadata,
            thinking=utils.get_pydantic_model(thinking, Optional[models.Thinking]),
            service_tier=service_tier,
        )

        req = self._build_request(
            method="POST",
            path="/messages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.AnthropicMessageRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createMessage",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return unmarshal_json_response(
                models.AnthropicMessageResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, models.CreateMessageResponseBody
                ).data,
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res, http_res_text
            )
            raise errors.BadRequestError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.AuthenticationErrorData, http_res, http_res_text
            )
            raise errors.AuthenticationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "403", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.PermissionDeniedErrorData, http_res, http_res_text
            )
            raise errors.PermissionDeniedError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.RateLimitErrorData, http_res, http_res_text
            )
            raise errors.RateLimitError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res, http_res_text
            )
            raise errors.InternalServerError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)

        http_res_text = utils.stream_to_text(http_res)
        raise errors.R9SDefaultError(
            "Unexpected response received", http_res, http_res_text
        )

    @overload
    async def create_async(
        self,
        *,
        model: str,
        messages: Union[
            List[models.AnthropicMessageMessage],
            List[models.AnthropicMessageMessageTypedDict],
        ],
        system: Optional[str] = None,
        max_tokens: Optional[int] = None,
        stop_sequences: Optional[List[str]] = None,
        stream: Union[Literal[False], None] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[
            Union[List[models.AnthropicTool], List[models.AnthropicToolTypedDict]]
        ] = None,
        tool_choice: Optional[
            Union[
                models.AnthropicMessageRequestToolChoice,
                models.AnthropicMessageRequestToolChoiceTypedDict,
            ]
        ] = None,
        metadata: Optional[Dict[str, Any]] = None,
        thinking: Optional[Union[models.Thinking, models.ThinkingTypedDict]] = None,
        service_tier: Optional[models.AnthropicMessageRequestServiceTier] = "auto",
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AnthropicMessageResponse:
        r"""Create message (Claude native API)

        Create a message using Anthropic Claude's native API format, supports streaming

        :param model: Claude model name
        :param messages: Messages list, first message must be a user message
        :param system: System prompt
        :param max_tokens: Maximum number of output tokens (optional).
            If not provided, the relay service or API may use a default value.
            Different models have different maximum values.

        :param stop_sequences: Stop sequences
        :param stream:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter. Only sample from the top K options for each subsequent token.
        :param tools:
        :param tool_choice:
        :param metadata: An object describing metadata about the request. Can be used for tracking, identification, or filtering purposes.
            Common use cases: user_id, session_id, request_id, etc.

        :param thinking: Configuration for extended thinking (Claude 3.7+). When enabled, the model will spend more time thinking before responding.

        :param service_tier: Service tier for request processing:
            - auto: Automatically select between standard and priority capacity
            - standard_only: Only use standard capacity (may have longer wait times during high load)

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    async def create_async(
        self,
        *,
        model: str,
        messages: Union[
            List[models.AnthropicMessageMessage],
            List[models.AnthropicMessageMessageTypedDict],
        ],
        system: Optional[str] = None,
        max_tokens: Optional[int] = None,
        stop_sequences: Optional[List[str]] = None,
        stream: Literal[True],
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[
            Union[List[models.AnthropicTool], List[models.AnthropicToolTypedDict]]
        ] = None,
        tool_choice: Optional[
            Union[
                models.AnthropicMessageRequestToolChoice,
                models.AnthropicMessageRequestToolChoiceTypedDict,
            ]
        ] = None,
        metadata: Optional[Dict[str, Any]] = None,
        thinking: Optional[Union[models.Thinking, models.ThinkingTypedDict]] = None,
        service_tier: Optional[models.AnthropicMessageRequestServiceTier] = "auto",
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[models.AnthropicStreamEvent]:
        r"""Create message (Claude native API)

        Create a message using Anthropic Claude's native API format, supports streaming

        :param model: Claude model name
        :param messages: Messages list, first message must be a user message
        :param system: System prompt
        :param max_tokens: Maximum number of output tokens (optional).
            If not provided, the relay service or API may use a default value.
            Different models have different maximum values.

        :param stop_sequences: Stop sequences
        :param stream:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter. Only sample from the top K options for each subsequent token.
        :param tools:
        :param tool_choice:
        :param metadata: An object describing metadata about the request. Can be used for tracking, identification, or filtering purposes.
            Common use cases: user_id, session_id, request_id, etc.

        :param thinking: Configuration for extended thinking (Claude 3.7+). When enabled, the model will spend more time thinking before responding.

        :param service_tier: Service tier for request processing:
            - auto: Automatically select between standard and priority capacity
            - standard_only: Only use standard capacity (may have longer wait times during high load)

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    async def create_async(
        self,
        *,
        model: str,
        messages: Union[
            List[models.AnthropicMessageMessage],
            List[models.AnthropicMessageMessageTypedDict],
        ],
        system: Optional[str] = None,
        max_tokens: Optional[int] = None,
        stop_sequences: Optional[List[str]] = None,
        stream: Optional[bool] = False,
        temperature: Optional[float] = None,
        top_p: Optional[float] = None,
        top_k: Optional[int] = None,
        tools: Optional[
            Union[List[models.AnthropicTool], List[models.AnthropicToolTypedDict]]
        ] = None,
        tool_choice: Optional[
            Union[
                models.AnthropicMessageRequestToolChoice,
                models.AnthropicMessageRequestToolChoiceTypedDict,
            ]
        ] = None,
        metadata: Optional[Dict[str, Any]] = None,
        thinking: Optional[Union[models.Thinking, models.ThinkingTypedDict]] = None,
        service_tier: Optional[models.AnthropicMessageRequestServiceTier] = "auto",
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateMessageResponse:
        r"""Create message (Claude native API)

        Create a message using Anthropic Claude's native API format, supports streaming

        :param model: Claude model name
        :param messages: Messages list, first message must be a user message
        :param system: System prompt
        :param max_tokens: Maximum number of output tokens (optional).
            If not provided, the relay service or API may use a default value.
            Different models have different maximum values.

        :param stop_sequences: Stop sequences
        :param stream:
        :param temperature:
        :param top_p:
        :param top_k: Top-k sampling parameter. Only sample from the top K options for each subsequent token.
        :param tools:
        :param tool_choice:
        :param metadata: An object describing metadata about the request. Can be used for tracking, identification, or filtering purposes.
            Common use cases: user_id, session_id, request_id, etc.

        :param thinking: Configuration for extended thinking (Claude 3.7+). When enabled, the model will spend more time thinking before responding.

        :param service_tier: Service tier for request processing:
            - auto: Automatically select between standard and priority capacity
            - standard_only: Only use standard capacity (may have longer wait times during high load)

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AnthropicMessageRequest(
            model=model,
            messages=utils.get_pydantic_model(
                messages, List[models.AnthropicMessageMessage]
            ),
            system=system,
            max_tokens=max_tokens,
            stop_sequences=stop_sequences,
            stream=stream,
            temperature=temperature,
            top_p=top_p,
            top_k=top_k,
            tools=utils.get_pydantic_model(tools, Optional[List[models.AnthropicTool]]),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[models.AnthropicMessageRequestToolChoice]
            ),
            metadata=metadata,
            thinking=utils.get_pydantic_model(thinking, Optional[models.Thinking]),
            service_tier=service_tier,
        )

        req = self._build_request_async(
            method="POST",
            path="/messages",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.AnthropicMessageRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createMessage",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return unmarshal_json_response(
                models.AnthropicMessageResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, models.CreateMessageResponseBody
                ).data,
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestErrorData, http_res, http_res_text
            )
            raise errors.BadRequestError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.AuthenticationErrorData, http_res, http_res_text
            )
            raise errors.AuthenticationError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "403", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.PermissionDeniedErrorData, http_res, http_res_text
            )
            raise errors.PermissionDeniedError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.RateLimitErrorData, http_res, http_res_text
            )
            raise errors.RateLimitError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerErrorData, http_res, http_res_text
            )
            raise errors.InternalServerError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.R9SDefaultError("API error occurred", http_res, http_res_text)

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.R9SDefaultError(
            "Unexpected response received", http_res, http_res_text
        )



from __future__ import annotations

import argparse
import json
import sys
from typing import Any
from typing import Optional, cast

from r9s import models
from r9s.cli_tools.commands import (
    CommandConfig,
    delete_command,
    list_commands,
    load_command,
    save_command,
)
from r9s.cli_tools.template_renderer import RenderContext, render_template
from r9s.cli_tools.bots import load_bot
from r9s.cli_tools.config import (
    get_api_key,
    resolve_base_url,
    resolve_model,
    resolve_system_prompt,
)
from r9s.cli_tools.i18n import resolve_lang, t
from r9s.sdk import R9S
from r9s.cli_tools.ui.terminal import (
    FG_RED,
    FG_YELLOW,
    error,
    header,
    info,
    prompt_text,
    success,
    warning,
)
from r9s.cli_tools.ui.spinner import Spinner


def _require_name(name: Optional[str]) -> str:
    if name and name.strip():
        return name.strip()
    raise SystemExit("command name is required")


def _is_interactive() -> bool:
    return sys.stdin.isatty()


def _prompt_optional(message: str) -> Optional[str]:
    value = prompt_text(message).strip()
    return value or None


def _prompt_multiline_required(message: str, *, hint: Optional[str] = None) -> str:
    header(message)
    if hint:
        info(hint)
    first = prompt_text("> ", color=FG_YELLOW)
    while not first:
        first = prompt_text("> ", color=FG_RED)
    lines = [first]
    while True:
        line = prompt_text("> ", color=FG_YELLOW)
        if not line:
            break
        lines.append(line)
    out = "\n".join(lines).rstrip()
    if not out:
        raise SystemExit("prompt cannot be empty")
    return out


def handle_command_list(_: argparse.Namespace) -> None:
    cmds = list_commands()
    if not cmds:
        info("No commands found.")
        return
    header("Commands")
    for c in cmds:
        print(f"- {c}")


def handle_command_show(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    cmd = load_command(name)
    header(f"Command: {cmd.name}")
    if cmd.description:
        print(f"- description: {cmd.description}")
    print("- prompt: (set)")


def handle_command_delete(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    confirm = prompt_text(f"Delete command '{name}'? [y/N]: ", color=FG_RED).lower()
    if confirm not in ("y", "yes"):
        warning("Cancelled.")
        return
    try:
        path = delete_command(name)
    except FileNotFoundError:
        error("Command not found.")
        return
    success(f"Deleted: {path}")


def handle_command_create(args: argparse.Namespace) -> None:
    name = _require_name(args.name)

    description = (args.description or "").strip() or None
    if description is None and _is_interactive():
        description = _prompt_optional("Description (optional): ")

    prompt: Optional[str] = None
    if args.prompt is not None:
        prompt = args.prompt.strip() or None
    elif args.prompt_file:
        prompt = args.prompt_file.read_text(encoding="utf-8").strip() or None
    elif _is_interactive():
        prompt = _prompt_multiline_required(
            "Prompt template (end with empty line):",
            hint="Template syntax: {{args}} and !{...}.",
        )

    if not prompt:
        raise SystemExit(
            "prompt is required (pass --prompt/--prompt-file or input interactively)"
        )

    cmd = CommandConfig(name=name, description=description, prompt=prompt)
    path = save_command(cmd)
    success(f"Saved: {path}")


def _read_stdin() -> str:
    if sys.stdin.isatty():
        return ""
    return sys.stdin.read()


def handle_command_render(args: argparse.Namespace) -> None:
    name = _require_name(args.name)
    cmd = load_command(name)
    args_text = " ".join(args.args or []).strip()
    _ = _read_stdin()  # stdin is currently unused by spec, but may be used by shell.
    rendered = render_template(
        cmd.prompt or "",
        RenderContext(
            args_text=args_text,
            assume_yes=bool(args.yes),
            interactive=sys.stdin.isatty(),
        ),
    )
    print(rendered)


def handle_command_run(args: argparse.Namespace) -> None:
    lang = resolve_lang(getattr(args, "lang", None))
    api_key = get_api_key(getattr(args, "api_key", None))
    if not api_key:
        raise SystemExit(t("chat.err.missing_api_key", lang))

    base_url = resolve_base_url(getattr(args, "base_url", None))
    model = resolve_model(getattr(args, "model", None))
    if not model:
        raise SystemExit(t("chat.err.missing_model", lang))

    system_prompt = resolve_system_prompt(None)
    temperature = None
    top_p = None
    max_tokens = None
    presence_penalty = None
    frequency_penalty = None
    if getattr(args, "bot", None):
        bot = load_bot(args.bot)
        if bot.system_prompt:
            system_prompt = bot.system_prompt
        temperature = bot.temperature
        top_p = bot.top_p
        max_tokens = bot.max_tokens
        presence_penalty = bot.presence_penalty
        frequency_penalty = bot.frequency_penalty

    cmd = load_command(_require_name(args.name))
    args_text = " ".join(args.args or []).strip()
    _ = _read_stdin()  # allow shell commands to read stdin if needed
    prompt = render_template(
        cmd.prompt or "",
        RenderContext(
            args_text=args_text,
            assume_yes=bool(getattr(args, "yes", False)),
            interactive=sys.stdin.isatty(),
        ),
    ).strip()
    if not prompt:
        raise SystemExit("Command produced empty prompt.")

    messages: list[models.MessageTypedDict] = []
    if system_prompt:
        system_prompt = render_template(
            system_prompt,
            RenderContext(
                args_text="",
                assume_yes=bool(getattr(args, "yes", False)),
                interactive=sys.stdin.isatty(),
            ),
        ).strip() or None
        messages.append(
            cast(models.MessageTypedDict, {"role": "system", "content": system_prompt})
        )
    messages.append(cast(models.MessageTypedDict, {"role": "user", "content": prompt}))

    with R9S(api_key=api_key, server_url=base_url) as r9s:
        if getattr(args, "no_stream", False):
            spinner = Spinner("")
            if sys.stdout.isatty():
                spinner.start()
            try:
                res = r9s.chat.create(
                    model=model,
                    messages=messages,
                    stream=False,
                    temperature=temperature,
                    top_p=top_p,
                    max_tokens=max_tokens,
                    presence_penalty=presence_penalty,
                    frequency_penalty=frequency_penalty,
                )
                text = ""
                if res.choices and res.choices[0].message:
                    text = _content_to_text(res.choices[0].message.content)
            finally:
                spinner.stop_and_clear()
            print(text)
            return

        spinner = Spinner("")
        if sys.stdout.isatty():
            spinner.start()
        stream = r9s.chat.create(
            model=model,
            messages=messages,
            stream=True,
            temperature=temperature,
            top_p=top_p,
            max_tokens=max_tokens,
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
        )
        try:
            parts: list[str] = []
            for event in stream:
                if not event.choices:
                    continue
                delta = event.choices[0].delta
                if delta.content:
                    spinner.stop_and_clear()
                    parts.append(delta.content)
                    print(delta.content, end="", flush=True)
        finally:
            spinner.stop_and_clear()
        print()


def _content_to_text(content: Any) -> str:
    if content is None:
        return ""
    if isinstance(content, str):
        return content
    if isinstance(content, list):
        parts: list[str] = []
        for item in content:
            if isinstance(item, dict):
                if item.get("type") == "text" and isinstance(item.get("text"), str):
                    parts.append(item["text"])
            else:
                item_type = getattr(item, "type", None)
                item_text = getattr(item, "text", None)
                if item_type == "text" and isinstance(item_text, str):
                    parts.append(item_text)
        if parts:
            return "".join(parts)
    try:
        return json.dumps(content, ensure_ascii=False)
    except Exception:
        return str(content)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .anthropiccontentblockdelta import (
    AnthropicContentBlockDelta,
    AnthropicContentBlockDeltaTypedDict,
)
from .anthropiccontentblockstart import (
    AnthropicContentBlockStart,
    AnthropicContentBlockStartTypedDict,
)
from .anthropiccontentblockstop import (
    AnthropicContentBlockStop,
    AnthropicContentBlockStopTypedDict,
)
from .anthropicmessagedelta import AnthropicMessageDelta, AnthropicMessageDeltaTypedDict
from .anthropicmessagestart import AnthropicMessageStart, AnthropicMessageStartTypedDict
from .anthropicping import AnthropicPing, AnthropicPingTypedDict
from .anthropicstreammessagestop import (
    AnthropicStreamMessageStop,
    AnthropicStreamMessageStopTypedDict,
)
from pydantic import Discriminator, Tag
from r9s.utils import get_discriminator
from typing import Union
from typing_extensions import Annotated, TypeAliasType


AnthropicStreamEventTypedDict = TypeAliasType(
    "AnthropicStreamEventTypedDict",
    Union[
        AnthropicStreamMessageStopTypedDict,
        AnthropicPingTypedDict,
        AnthropicMessageStartTypedDict,
        AnthropicContentBlockStartTypedDict,
        AnthropicContentBlockDeltaTypedDict,
        AnthropicContentBlockStopTypedDict,
        AnthropicMessageDeltaTypedDict,
    ],
)


AnthropicStreamEvent = Annotated[
    Union[
        Annotated[AnthropicMessageStart, Tag("message_start")],
        Annotated[AnthropicContentBlockStart, Tag("content_block_start")],
        Annotated[AnthropicContentBlockDelta, Tag("content_block_delta")],
        Annotated[AnthropicContentBlockStop, Tag("content_block_stop")],
        Annotated[AnthropicMessageDelta, Tag("message_delta")],
        Annotated[AnthropicStreamMessageStop, Tag("message_stop")],
        Annotated[AnthropicPing, Tag("ping")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Tests for image generation functionality."""

from __future__ import annotations

from r9s.models.imagegenerationrequest import (
    ImageGenerationRequest,
    ImageGenerationRequestTypedDict,
)
from r9s.models.imagegenerationresponse import (
    ImageGenerationResponse,
    ImageUsage,
    ImageUsageTypedDict,
)
from r9s.models.imageobject import ImageObject


class TestImageGenerationRequestModel:
    """Tests for ImageGenerationRequest model."""

    def test_minimal_request(self) -> None:
        """Request with only required fields."""
        request = ImageGenerationRequest(prompt="A beautiful sunset")
        assert request.prompt == "A beautiful sunset"
        assert request.model is None
        assert request.n == 1
        assert request.quality == "standard"
        assert request.response_format == "url"
        assert request.size == "1024x1024"
        assert request.style == "vivid"
        assert request.user is None
        # Extended parameters default to None
        assert request.negative_prompt is None
        assert request.seed is None
        assert request.prompt_extend is None
        assert request.watermark is None

    def test_full_request_with_extended_params(self) -> None:
        """Request with all fields including extended parameters."""
        request = ImageGenerationRequest(
            prompt="A serene mountain landscape",
            model="wanx-v1",
            n=1,
            quality="hd",
            response_format="b64_json",
            size="1024x1024",
            style="natural",
            user="user-123",
            negative_prompt="people, buildings, cars",
            seed=12345,
            prompt_extend=True,
            watermark=False,
        )
        assert request.prompt == "A serene mountain landscape"
        assert request.model == "wanx-v1"
        assert request.quality == "hd"
        assert request.negative_prompt == "people, buildings, cars"
        assert request.seed == 12345
        assert request.prompt_extend is True
        assert request.watermark is False

    def test_negative_prompt_parameter(self) -> None:
        """negative_prompt parameter works correctly."""
        request = ImageGenerationRequest(
            prompt="A forest",
            negative_prompt="fog, mist, rain",
        )
        assert request.negative_prompt == "fog, mist, rain"

    def test_seed_parameter(self) -> None:
        """seed parameter works correctly."""
        request = ImageGenerationRequest(
            prompt="A blue dragon",
            seed=42,
        )
        assert request.seed == 42

    def test_seed_max_value(self) -> None:
        """seed parameter accepts max Qwen value."""
        request = ImageGenerationRequest(
            prompt="Test",
            seed=2147483647,
        )
        assert request.seed == 2147483647

    def test_prompt_extend_parameter(self) -> None:
        """prompt_extend parameter works correctly."""
        request = ImageGenerationRequest(
            prompt="A cat",
            prompt_extend=True,
        )
        assert request.prompt_extend is True

        request = ImageGenerationRequest(
            prompt="A cat",
            prompt_extend=False,
        )
        assert request.prompt_extend is False

    def test_watermark_parameter(self) -> None:
        """watermark parameter works correctly."""
        request = ImageGenerationRequest(
            prompt="A dog",
            watermark=True,
        )
        assert request.watermark is True

        request = ImageGenerationRequest(
            prompt="A dog",
            watermark=False,
        )
        assert request.watermark is False


class TestImageGenerationRequestSizes:
    """Tests for Size literal values."""

    def test_standard_sizes(self) -> None:
        """Standard OpenAI sizes are accepted."""
        standard_sizes = ["256x256", "512x512", "1024x1024"]
        for size in standard_sizes:
            req = ImageGenerationRequest(prompt="test", size=size)  # type: ignore
            assert req.size == size

    def test_dalle3_sizes(self) -> None:
        """DALL-E 3 sizes are accepted."""
        dalle3_sizes = ["1792x1024", "1024x1792"]
        for size in dalle3_sizes:
            req = ImageGenerationRequest(prompt="test", size=size)  # type: ignore
            assert req.size == size

    def test_gpt_image_sizes(self) -> None:
        """GPT-Image-1/1.5 sizes are accepted."""
        gpt_sizes = ["1024x1536", "1536x1024"]
        for size in gpt_sizes:
            req = ImageGenerationRequest(prompt="test", size=size)  # type: ignore
            assert req.size == size

    def test_wanx_sizes(self) -> None:
        """Wanx/Ali sizes are accepted."""
        wanx_sizes = ["720x1280", "1280x720", "512x1024", "1024x768", "576x1024", "1024x576"]
        for size in wanx_sizes:
            req = ImageGenerationRequest(prompt="test", size=size)  # type: ignore
            assert req.size == size

    def test_gemini_aspect_ratios(self) -> None:
        """Gemini Nano Banana aspect ratios are accepted."""
        aspect_ratios = ["1:1", "16:9", "9:16", "4:3", "3:4", "3:2", "2:3", "5:4", "4:5", "21:9"]
        for ratio in aspect_ratios:
            req = ImageGenerationRequest(prompt="test", size=ratio)  # type: ignore
            assert req.size == ratio


class TestImageGenerationRequestQuality:
    """Tests for Quality literal values."""

    def test_standard_quality_values(self) -> None:
        """Standard quality values are accepted."""
        for quality in ["standard", "hd"]:
            req = ImageGenerationRequest(prompt="test", quality=quality)  # type: ignore
            assert req.quality == quality

    def test_extended_quality_values(self) -> None:
        """Extended GPT-Image quality values are accepted."""
        for quality in ["low", "medium", "high"]:
            req = ImageGenerationRequest(prompt="test", quality=quality)  # type: ignore
            assert req.quality == quality


class TestImageGenerationRequestTypedDict:
    """Tests for ImageGenerationRequestTypedDict."""

    def test_typed_dict_minimal(self) -> None:
        """TypedDict with only required fields."""
        data: ImageGenerationRequestTypedDict = {
            "prompt": "A beautiful sunset",
        }
        assert data["prompt"] == "A beautiful sunset"

    def test_typed_dict_with_extended_params(self) -> None:
        """TypedDict with extended parameters."""
        data: ImageGenerationRequestTypedDict = {
            "prompt": "A mountain landscape",
            "model": "wanx-v1",
            "negative_prompt": "people, buildings",
            "seed": 12345,
            "prompt_extend": True,
            "watermark": False,
        }
        assert data["negative_prompt"] == "people, buildings"
        assert data["seed"] == 12345
        assert data["prompt_extend"] is True
        assert data["watermark"] is False


class TestImageUsageModel:
    """Tests for ImageUsage model."""

    def test_empty_usage(self) -> None:
        """ImageUsage with no fields set."""
        usage = ImageUsage()
        assert usage.prompt_tokens is None
        assert usage.image_tokens is None
        assert usage.input_text_tokens is None
        assert usage.output_image_tokens is None
        assert usage.width is None
        assert usage.height is None
        assert usage.image_count is None

    def test_openai_usage(self) -> None:
        """ImageUsage with OpenAI-style fields."""
        usage = ImageUsage(
            prompt_tokens=100,
            image_tokens=500,
        )
        assert usage.prompt_tokens == 100
        assert usage.image_tokens == 500

    def test_qwen_usage(self) -> None:
        """ImageUsage with Qwen-style fields."""
        usage = ImageUsage(
            input_text_tokens=50,
            output_image_tokens=1000,
            width=1024,
            height=1024,
            image_count=1,
        )
        assert usage.input_text_tokens == 50
        assert usage.output_image_tokens == 1000
        assert usage.width == 1024
        assert usage.height == 1024
        assert usage.image_count == 1

    def test_full_usage(self) -> None:
        """ImageUsage with all fields populated."""
        usage = ImageUsage(
            prompt_tokens=100,
            image_tokens=500,
            input_text_tokens=50,
            output_image_tokens=1000,
            width=1024,
            height=768,
            image_count=2,
        )
        assert usage.prompt_tokens == 100
        assert usage.image_tokens == 500
        assert usage.input_text_tokens == 50
        assert usage.output_image_tokens == 1000
        assert usage.width == 1024
        assert usage.height == 768
        assert usage.image_count == 2


class TestImageUsageTypedDict:
    """Tests for ImageUsageTypedDict."""

    def test_typed_dict_empty(self) -> None:
        """TypedDict with no fields."""
        data: ImageUsageTypedDict = {}
        assert len(data) == 0

    def test_typed_dict_full(self) -> None:
        """TypedDict with all fields."""
        data: ImageUsageTypedDict = {
            "prompt_tokens": 100,
            "image_tokens": 500,
            "input_text_tokens": 50,
            "output_image_tokens": 1000,
            "width": 1024,
            "height": 768,
            "image_count": 2,
        }
        assert data["prompt_tokens"] == 100
        assert data["image_tokens"] == 500


class TestImageGenerationResponseModel:
    """Tests for ImageGenerationResponse model."""

    def test_response_without_usage(self) -> None:
        """Response without usage information."""
        response = ImageGenerationResponse(
            created=1234567890,
            data=[ImageObject(url="https://example.com/image.png")],
        )
        assert response.created == 1234567890
        assert len(response.data) == 1
        assert response.data[0].url == "https://example.com/image.png"
        assert response.usage is None

    def test_response_with_usage(self) -> None:
        """Response with usage information."""
        usage = ImageUsage(prompt_tokens=100, image_tokens=500)
        response = ImageGenerationResponse(
            created=1234567890,
            data=[ImageObject(url="https://example.com/image.png")],
            usage=usage,
        )
        assert response.usage is not None
        assert response.usage.prompt_tokens == 100
        assert response.usage.image_tokens == 500

    def test_response_with_multiple_images(self) -> None:
        """Response with multiple images (gpt-image-1.5 style)."""
        response = ImageGenerationResponse(
            created=1234567890,
            data=[
                ImageObject(url="https://example.com/image1.png"),
                ImageObject(url="https://example.com/image2.png"),
                ImageObject(url="https://example.com/image3.png"),
            ],
        )
        assert len(response.data) == 3

    def test_response_with_b64_json(self) -> None:
        """Response with base64 encoded images."""
        response = ImageGenerationResponse(
            created=1234567890,
            data=[ImageObject(b64_json="iVBORw0KGgoAAAANSUhEUgAAAAEAAAAB...")],
        )
        assert response.data[0].b64_json is not None
        assert response.data[0].url is None

    def test_response_with_revised_prompt(self) -> None:
        """Response includes revised_prompt from DALL-E 3."""
        response = ImageGenerationResponse(
            created=1234567890,
            data=[
                ImageObject(
                    url="https://example.com/image.png",
                    revised_prompt="An enhanced description of the sunset",
                )
            ],
        )
        assert response.data[0].revised_prompt == "An enhanced description of the sunset"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .anthropicinputschema import AnthropicInputSchema, AnthropicInputSchemaTypedDict
from r9s.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class AnthropicToolTypedDict(TypedDict):
    name: str
    input_schema: AnthropicInputSchemaTypedDict
    description: NotRequired[str]


class AnthropicTool(BaseModel):
    name: str

    input_schema: AnthropicInputSchema

    description: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .message import Message, MessageTypedDict
from .responsetool import ResponseTool, ResponseToolTypedDict
from .toolchoice import ToolChoice, ToolChoiceTypedDict
import pydantic
from r9s.types import BaseModel
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


InputTypedDict = TypeAliasType("InputTypedDict", Union[str, List[MessageTypedDict]])
r"""Input content, required parameter. Can be:
- String: Single text input
- Message array: Structured conversation history

**Important limitations:**
- Messages only support basic fields (role, content, name)
- Does not support tool_calls, tool_call_id and other tool-related fields
- content field is required and cannot be null
- To use tools, define them in the top-level tools parameter; model will call them on first response

Note: Responses API has deprecated messages parameter, now uses input parameter uniformly

"""


Input = TypeAliasType("Input", Union[str, List[Message]])
r"""Input content, required parameter. Can be:
- String: Single text input
- Message array: Structured conversation history

**Important limitations:**
- Messages only support basic fields (role, content, name)
- Does not support tool_calls, tool_call_id and other tool-related fields
- content field is required and cannot be null
- To use tools, define them in the top-level tools parameter; model will call them on first response

Note: Responses API has deprecated messages parameter, now uses input parameter uniformly

"""


ResponseRequestModalities = Literal[
    "text",
    "audio",
]


ResponseRequest1 = Literal[
    "none",
    "auto",
    "required",
]


ResponseRequestToolChoiceTypedDict = TypeAliasType(
    "ResponseRequestToolChoiceTypedDict", Union[ToolChoiceTypedDict, ResponseRequest1]
)
r"""Tool selection strategy"""


ResponseRequestToolChoice = TypeAliasType(
    "ResponseRequestToolChoice", Union[ToolChoice, ResponseRequest1]
)
r"""Tool selection strategy"""


ResponseRequestType = Literal[
    "text",
    "json_object",
    "json_schema",
]
r"""The type of response format"""


class ResponseRequestFormatTypedDict(TypedDict):
    r"""An object specifying the format that the model must output. Setting to { \"type\": \"json_schema\", \"name\": \"...\", \"schema\": {...} } enables Structured Outputs which ensures the model will match your supplied JSON schema.
    Setting to { \"type\": \"json_object\" } enables JSON mode, which ensures the model generates valid JSON.

    """

    type: NotRequired[ResponseRequestType]
    r"""The type of response format"""
    name: NotRequired[str]
    r"""Name for the schema (required when type is json_schema)"""
    schema_: NotRequired[Dict[str, Any]]
    r"""JSON schema definition for structured outputs"""
    strict: NotRequired[bool]
    r"""Whether to enforce strict schema matching"""


class ResponseRequestFormat(BaseModel):
    r"""An object specifying the format that the model must output. Setting to { \"type\": \"json_schema\", \"name\": \"...\", \"schema\": {...} } enables Structured Outputs which ensures the model will match your supplied JSON schema.
    Setting to { \"type\": \"json_object\" } enables JSON mode, which ensures the model generates valid JSON.

    """

    type: Optional[ResponseRequestType] = None
    r"""The type of response format"""

    name: Optional[str] = None
    r"""Name for the schema (required when type is json_schema)"""

    schema_: Annotated[Optional[Dict[str, Any]], pydantic.Field(alias="schema")] = None
    r"""JSON schema definition for structured outputs"""

    strict: Optional[bool] = None
    r"""Whether to enforce strict schema matching"""


Verbosity = Literal[
    "low",
    "medium",
    "high",
]
r"""Verbosity level for the text output"""


class TextTypedDict(TypedDict):
    r"""Text output configuration"""

    format_: NotRequired[ResponseRequestFormatTypedDict]
    r"""An object specifying the format that the model must output. Setting to { \"type\": \"json_schema\", \"name\": \"...\", \"schema\": {...} } enables Structured Outputs which ensures the model will match your supplied JSON schema.
    Setting to { \"type\": \"json_object\" } enables JSON mode, which ensures the model generates valid JSON.

    """
    verbosity: NotRequired[Verbosity]
    r"""Verbosity level for the text output"""


class Text(BaseModel):
    r"""Text output configuration"""

    format_: Annotated[
        Optional[ResponseRequestFormat], pydantic.Field(alias="format")
    ] = None
    r"""An object specifying the format that the model must output. Setting to { \"type\": \"json_schema\", \"name\": \"...\", \"schema\": {...} } enables Structured Outputs which ensures the model will match your supplied JSON schema.
    Setting to { \"type\": \"json_object\" } enables JSON mode, which ensures the model generates valid JSON.

    """

    verbosity: Optional[Verbosity] = None
    r"""Verbosity level for the text output"""


Effort = Literal[
    "none",
    "minimal",
    "low",
    "medium",
    "high",
    "xhigh",
]
r"""The effort level for reasoning (none/minimal=fast, low/medium=balanced, high/xhigh=thorough)"""


class ReasoningTypedDict(TypedDict):
    r"""Configuration for reasoning models (e.g., o1, o3, gpt-5). Controls how the model uses reasoning tokens to \"think\" through the problem."""

    effort: NotRequired[Effort]
    r"""The effort level for reasoning (none/minimal=fast, low/medium=balanced, high/xhigh=thorough)"""
    summary: NotRequired[str]
    r"""Summary of reasoning approach"""


class Reasoning(BaseModel):
    r"""Configuration for reasoning models (e.g., o1, o3, gpt-5). Controls how the model uses reasoning tokens to \"think\" through the problem."""

    effort: Optional[Effort] = None
    r"""The effort level for reasoning (none/minimal=fast, low/medium=balanced, high/xhigh=thorough)"""

    summary: Optional[str] = None
    r"""Summary of reasoning approach"""


Truncation = Literal[
    "auto",
    "disabled",
]
r"""The truncation strategy to use for the model response.
- auto: If input exceeds context window, truncate by dropping items from beginning
- disabled: Request fails with 400 error if input exceeds context window (default)

"""


ResponseRequestStopTypedDict = TypeAliasType(
    "ResponseRequestStopTypedDict", Union[str, List[str]]
)
r"""Up to 4 sequences where the API will stop generating further tokens"""


ResponseRequestStop = TypeAliasType("ResponseRequestStop", Union[str, List[str]])
r"""Up to 4 sequences where the API will stop generating further tokens"""


class ResponseRequestTypedDict(TypedDict):
    model: str
    r"""Model name"""
    input: InputTypedDict
    r"""Input content, required parameter. Can be:
    - String: Single text input
    - Message array: Structured conversation history

    **Important limitations:**
    - Messages only support basic fields (role, content, name)
    - Does not support tool_calls, tool_call_id and other tool-related fields
    - content field is required and cannot be null
    - To use tools, define them in the top-level tools parameter; model will call them on first response

    Note: Responses API has deprecated messages parameter, now uses input parameter uniformly

    """
    instructions: NotRequired[str]
    r"""System-level instructions to guide model behavior and response style (similar to system message)"""
    temperature: NotRequired[float]
    r"""Controls output randomness, higher values mean more random"""
    top_p: NotRequired[float]
    r"""Nucleus sampling parameter, controls output diversity"""
    max_output_tokens: NotRequired[int]
    r"""Maximum number of tokens to generate"""
    stream: NotRequired[bool]
    r"""Whether to enable streaming"""
    modalities: NotRequired[List[ResponseRequestModalities]]
    r"""Response modality types"""
    tools: NotRequired[List[ResponseToolTypedDict]]
    r"""Available tools list (using flat format)"""
    tool_choice: NotRequired[ResponseRequestToolChoiceTypedDict]
    r"""Tool selection strategy"""
    parallel_tool_calls: NotRequired[bool]
    r"""Whether to enable parallel function calling during tool use. When false, ensures exactly zero or one tool is called."""
    text: NotRequired[TextTypedDict]
    r"""Text output configuration"""
    previous_response_id: NotRequired[str]
    r"""The ID of a previous response to continue the conversation from. This allows you to chain responses together and maintain conversation state.
    When using previous_response_id, the model will automatically have access to all previously produced reasoning items and conversation history.

    """
    store: NotRequired[bool]
    r"""Whether to store the generated model response for later retrieval via API.
    Defaults to true. Set to false to disable storage (required for ZDR organizations).

    """
    background: NotRequired[bool]
    r"""Whether to run the model response in the background asynchronously. Useful for long-running tasks."""
    reasoning: NotRequired[ReasoningTypedDict]
    r"""Configuration for reasoning models (e.g., o1, o3, gpt-5). Controls how the model uses reasoning tokens to \"think\" through the problem."""
    truncation: NotRequired[Truncation]
    r"""The truncation strategy to use for the model response.
    - auto: If input exceeds context window, truncate by dropping items from beginning
    - disabled: Request fails with 400 error if input exceeds context window (default)

    """
    stop: NotRequired[ResponseRequestStopTypedDict]
    r"""Up to 4 sequences where the API will stop generating further tokens"""
    metadata: NotRequired[Dict[str, Any]]
    r"""Additional metadata for tracking and organization purposes"""


class ResponseRequest(BaseModel):
    model: str
    r"""Model name"""

    input: Input
    r"""Input content, required parameter. Can be:
    - String: Single text input
    - Message array: Structured conversation history

    **Important limitations:**
    - Messages only support basic fields (role, content, name)
    - Does not support tool_calls, tool_call_id and other tool-related fields
    - content field is required and cannot be null
    - To use tools, define them in the top-level tools parameter; model will call them on first response

    Note: Responses API has deprecated messages parameter, now uses input parameter uniformly

    """

    instructions: Optional[str] = None
    r"""System-level instructions to guide model behavior and response style (similar to system message)"""

    temperature: Optional[float] = None
    r"""Controls output randomness, higher values mean more random"""

    top_p: Optional[float] = None
    r"""Nucleus sampling parameter, controls output diversity"""

    max_output_tokens: Optional[int] = None
    r"""Maximum number of tokens to generate"""

    stream: Optional[bool] = False
    r"""Whether to enable streaming"""

    modalities: Optional[List[ResponseRequestModalities]] = None
    r"""Response modality types"""

    tools: Optional[List[ResponseTool]] = None
    r"""Available tools list (using flat format)"""

    tool_choice: Optional[ResponseRequestToolChoice] = None
    r"""Tool selection strategy"""

    parallel_tool_calls: Optional[bool] = True
    r"""Whether to enable parallel function calling during tool use. When false, ensures exactly zero or one tool is called."""

    text: Optional[Text] = None
    r"""Text output configuration"""

    previous_response_id: Optional[str] = None
    r"""The ID of a previous response to continue the conversation from. This allows you to chain responses together and maintain conversation state.
    When using previous_response_id, the model will automatically have access to all previously produced reasoning items and conversation history.

    """

    store: Optional[bool] = True
    r"""Whether to store the generated model response for later retrieval via API.
    Defaults to true. Set to false to disable storage (required for ZDR organizations).

    """

    background: Optional[bool] = False
    r"""Whether to run the model response in the background asynchronously. Useful for long-running tasks."""

    reasoning: Optional[Reasoning] = None
    r"""Configuration for reasoning models (e.g., o1, o3, gpt-5). Controls how the model uses reasoning tokens to \"think\" through the problem."""

    truncation: Optional[Truncation] = "disabled"
    r"""The truncation strategy to use for the model response.
    - auto: If input exceeds context window, truncate by dropping items from beginning
    - disabled: Request fails with 400 error if input exceeds context window (default)

    """

    stop: Optional[ResponseRequestStop] = None
    r"""Up to 4 sequences where the API will stop generating further tokens"""

    metadata: Optional[Dict[str, Any]] = None
    r"""Additional metadata for tracking and organization purposes"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcompletionstreamchoice import (
    ChatCompletionStreamChoice,
    ChatCompletionStreamChoiceTypedDict,
)
from .usage import Usage, UsageTypedDict
import pydantic
from pydantic.functional_validators import AfterValidator
from r9s.types import BaseModel
from r9s.utils import validate_const
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChatCompletionStreamEventTypedDict(TypedDict):
    r"""Chat completion chunk event (data payload for SSE stream)"""

    id: str
    created: int
    r"""Unix timestamp"""
    model: str
    choices: List[ChatCompletionStreamChoiceTypedDict]
    r"""Array of completion choices. May be empty in the final chunk when only usage is returned."""
    object: Literal["chat.completion.chunk"]
    system_fingerprint: NotRequired[str]
    usage: NotRequired[UsageTypedDict]


class ChatCompletionStreamEvent(BaseModel):
    r"""Chat completion chunk event (data payload for SSE stream)"""

    id: str

    created: int
    r"""Unix timestamp"""

    model: str

    choices: List[ChatCompletionStreamChoice]
    r"""Array of completion choices. May be empty in the final chunk when only usage is returned."""

    OBJECT: Annotated[
        Annotated[
            Literal["chat.completion.chunk"],
            AfterValidator(validate_const("chat.completion.chunk")),
        ],
        pydantic.Field(alias="object"),
    ] = "chat.completion.chunk"

    system_fingerprint: Optional[str] = None

    usage: Optional[Usage] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .imagegenerationresponse import ImageUsage, ImageUsageTypedDict
from r9s.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ImageGenerationStreamDataTypedDict(TypedDict):
    """Image generation streaming data - single image in the stream"""

    index: int
    r"""Image index in the batch"""

    b64_json: NotRequired[str]
    r"""Base64-encoded partial or final image"""

    url: NotRequired[str]
    r"""Image URL (alternative to b64_json)"""

    progress: NotRequired[float]
    r"""Generation progress (0.0-1.0)"""

    is_final: NotRequired[bool]
    r"""Whether this is the final image"""

    revised_prompt: NotRequired[str]
    r"""Revised prompt (if applicable)"""


class ImageGenerationStreamData(BaseModel):
    """Image generation streaming data - single image in the stream"""

    index: int
    r"""Image index in the batch"""

    b64_json: Optional[str] = None
    r"""Base64-encoded partial or final image"""

    url: Optional[str] = None
    r"""Image URL (alternative to b64_json)"""

    progress: Optional[float] = None
    r"""Generation progress (0.0-1.0)"""

    is_final: Optional[bool] = None
    r"""Whether this is the final image"""

    revised_prompt: Optional[str] = None
    r"""Revised prompt (if applicable)"""


class ImageGenerationStreamEventTypedDict(TypedDict):
    """Image generation streaming event - chunk in SSE stream"""

    id: str
    r"""Unique generation request identifier"""

    created: int
    r"""Unix timestamp"""

    model: str
    r"""Model used for generation"""

    object: str
    r"""Object type identifier (image.generation.chunk)"""

    data: List[ImageGenerationStreamDataTypedDict]
    r"""Array of image data chunks"""

    usage: NotRequired[ImageUsageTypedDict]
    r"""Usage statistics (only in final chunk)"""


class ImageGenerationStreamEvent(BaseModel):
    """Image generation streaming event - chunk in SSE stream"""

    id: str
    r"""Unique generation request identifier"""

    created: int
    r"""Unix timestamp"""

    model: str
    r"""Model used for generation"""

    object: str
    r"""Object type identifier (image.generation.chunk)"""

    data: List[ImageGenerationStreamData]
    r"""Array of image data chunks"""

    usage: Optional[ImageUsage] = None
    r"""Usage statistics (only in final chunk)"""



from __future__ import annotations

import base64

from r9s.cli_tools.chat_cli import _build_user_message_from_piped_stdin
from r9s.cli_tools.chat_extensions import ChatContext


def test_build_user_message_from_piped_stdin_png() -> None:
    # Minimal bytes with a valid PNG signature (parser only checks header).
    png_bytes = b"\x89PNG\r\n\x1a\n" + b"\x00" * 8
    raw = b"\n" + png_bytes + b"\n"

    ctx = ChatContext(base_url="", model="")
    msg = _build_user_message_from_piped_stdin(raw, lang="en", exts=[], ctx=ctx)
    assert msg is not None
    assert msg.get("role") == "user"
    content = msg.get("content")
    assert isinstance(content, list)

    first = content[0]
    assert isinstance(first, dict)
    assert first.get("type") == "text"
    assert first.get("text") == "Describe this image."

    second = content[1]
    assert isinstance(second, dict)
    assert second.get("type") == "image_url"
    image_url = second.get("image_url")
    assert isinstance(image_url, dict)
    url = image_url.get("url")
    assert isinstance(url, str)
    assert url.startswith("data:image/png;base64,")
    assert url == "data:image/png;base64," + base64.b64encode(png_bytes).decode("ascii")



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .anthropicimagesourcebase64 import (
    AnthropicImageSourceBase64,
    AnthropicImageSourceBase64TypedDict,
)
from .anthropicimagesourceurl import (
    AnthropicImageSourceURL,
    AnthropicImageSourceURLTypedDict,
)
from pydantic import Discriminator, Tag
from r9s.utils import get_discriminator
from typing import Union
from typing_extensions import Annotated, TypeAliasType


AnthropicImageSourceTypedDict = TypeAliasType(
    "AnthropicImageSourceTypedDict",
    Union[AnthropicImageSourceURLTypedDict, AnthropicImageSourceBase64TypedDict],
)


AnthropicImageSource = Annotated[
    Union[
        Annotated[AnthropicImageSourceBase64, Tag("base64")],
        Annotated[AnthropicImageSourceURL, Tag("url")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import io
import pydantic
from r9s.types import BaseModel
from r9s.utils import FieldMetadata, MultipartFormMetadata
from typing import IO, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class AudioTranslationRequestFileTypedDict(TypedDict):
    file_name: str
    content: Union[bytes, IO[bytes], io.BufferedReader]
    content_type: NotRequired[str]


class AudioTranslationRequestFile(BaseModel):
    file_name: Annotated[
        str, pydantic.Field(alias="fileName"), FieldMetadata(multipart=True)
    ]

    content: Annotated[
        Union[bytes, IO[bytes], io.BufferedReader],
        pydantic.Field(alias=""),
        FieldMetadata(multipart=MultipartFormMetadata(content=True)),
    ]

    content_type: Annotated[
        Optional[str],
        pydantic.Field(alias="Content-Type"),
        FieldMetadata(multipart=True),
    ] = None


AudioTranslationRequestResponseFormat = Literal[
    "json",
    "text",
    "srt",
    "verbose_json",
    "vtt",
]
r"""Output format for the translated text"""


class AudioTranslationRequestTypedDict(TypedDict):
    file: AudioTranslationRequestFileTypedDict
    r"""Audio file to translate to English"""
    model: str
    r"""Model name (whisper-1 is primary, gpt-4o-transcribe has extended support)"""
    prompt: NotRequired[str]
    r"""Optional text prompt to guide the model's style.
    The source language can be specified in the prompt if needed, though the model will auto-detect it.

    """
    response_format: NotRequired[AudioTranslationRequestResponseFormat]
    r"""Output format for the translated text"""
    temperature: NotRequired[float]
    r"""Sampling temperature between 0 and 1"""


class AudioTranslationRequest(BaseModel):
    file: Annotated[
        AudioTranslationRequestFile,
        FieldMetadata(multipart=MultipartFormMetadata(file=True)),
    ]
    r"""Audio file to translate to English"""

    model: Annotated[str, FieldMetadata(multipart=True)]
    r"""Model name (whisper-1 is primary, gpt-4o-transcribe has extended support)"""

    prompt: Annotated[Optional[str], FieldMetadata(multipart=True)] = None
    r"""Optional text prompt to guide the model's style.
    The source language can be specified in the prompt if needed, though the model will auto-detect it.

    """

    response_format: Annotated[
        Optional[AudioTranslationRequestResponseFormat], FieldMetadata(multipart=True)
    ] = "json"
    r"""Output format for the translated text"""

    temperature: Annotated[Optional[float], FieldMetadata(multipart=True)] = 0
    r"""Sampling temperature between 0 and 1"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .moderationcategories import ModerationCategories, ModerationCategoriesTypedDict
from .moderationcategoryscores import (
    ModerationCategoryScores,
    ModerationCategoryScoresTypedDict,
)
from r9s.types import BaseModel
from typing_extensions import TypedDict


class ModerationResultTypedDict(TypedDict):
    flagged: bool
    categories: ModerationCategoriesTypedDict
    category_scores: ModerationCategoryScoresTypedDict


class ModerationResult(BaseModel):
    flagged: bool

    categories: ModerationCategories

    category_scores: ModerationCategoryScores



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from pydantic.functional_validators import AfterValidator
from r9s.types import BaseModel
from r9s.utils import validate_const
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AnthropicThinkingContentTypedDict(TypedDict):
    thinking: str
    r"""The model's thinking process/reasoning"""
    type: Literal["thinking"]
    r"""Extended thinking output content"""
    signature: NotRequired[str]
    r"""Cryptographic signature for thinking content verification"""


class AnthropicThinkingContent(BaseModel):
    thinking: str
    r"""The model's thinking process/reasoning"""

    TYPE: Annotated[
        Annotated[Literal["thinking"], AfterValidator(validate_const("thinking"))],
        pydantic.Field(alias="type"),
    ] = "thinking"
    r"""Extended thinking output content"""

    signature: Optional[str] = None
    r"""Cryptographic signature for thinking content verification"""



from __future__ import annotations

import os
from typing import Optional


def get_api_key(args_api_key: Optional[str]) -> Optional[str]:
    value = (os.getenv("R9S_API_KEY") or args_api_key or "").strip()
    return value or None


def resolve_base_url(args_base_url: Optional[str]) -> str:
    return (
        args_base_url or os.getenv("R9S_BASE_URL") or "https://api.r9s.ai/v1"
    ).strip()


def is_valid_url(url: str) -> bool:
    """Check if URL format is valid (must start with http:// or https://)."""
    if not url:
        return False
    url = url.strip()
    return url.startswith("http://") or url.startswith("https://")


def resolve_model(args_model: Optional[str]) -> str:
    return (args_model or os.getenv("R9S_MODEL") or "gpt-5-nano").strip()


def resolve_image_model(args_model: Optional[str], default: str = "gpt-image-1.5") -> str:
    """Resolve image model: args > R9S_IMAGE_MODEL > default.

    Note: Does NOT fall back to R9S_MODEL since chat models are not valid for images.
    """
    return (
        args_model
        or os.getenv("R9S_IMAGE_MODEL")
        or default
    ).strip()


def resolve_tts_model(args_model: Optional[str], default: str = "tts-1") -> str:
    """Resolve TTS model: args > R9S_TTS_MODEL > default."""
    return (args_model or os.getenv("R9S_TTS_MODEL") or default).strip()


def resolve_stt_model(args_model: Optional[str], default: str = "whisper-1") -> str:
    """Resolve STT model: args > R9S_STT_MODEL > default."""
    return (args_model or os.getenv("R9S_STT_MODEL") or default).strip()


def resolve_system_prompt(
    args_system_prompt: Optional[str],
) -> Optional[str]:
    if args_system_prompt:
        val = args_system_prompt.strip()
        return val or None
    env = (os.getenv("R9S_SYSTEM_PROMPT") or "").strip()
    return env or None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from r9s.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EditRequestTypedDict(TypedDict):
    model: str
    r"""Model name"""
    instruction: str
    r"""Edit instruction"""
    input: NotRequired[str]
    r"""Input text to edit"""
    n: NotRequired[int]
    temperature: NotRequired[float]
    top_p: NotRequired[float]


class EditRequest(BaseModel):
    model: str
    r"""Model name"""

    instruction: str
    r"""Edit instruction"""

    input: Optional[str] = ""
    r"""Input text to edit"""

    n: Optional[int] = 1

    temperature: Optional[float] = 1

    top_p: Optional[float] = 1



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responseobject import ResponseObject, ResponseObjectTypedDict
from .responseoutputitem import ResponseOutputItem, ResponseOutputItemTypedDict
from r9s.types import BaseModel
from typing import Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


class ResponseStreamEvent2TypedDict(TypedDict):
    r"""Delta updates object format"""

    text: NotRequired[str]
    r"""Text delta content"""
    role: NotRequired[str]
    r"""Role delta"""
    content: NotRequired[str]
    r"""Content delta"""


class ResponseStreamEvent2(BaseModel):
    r"""Delta updates object format"""

    text: Optional[str] = None
    r"""Text delta content"""

    role: Optional[str] = None
    r"""Role delta"""

    content: Optional[str] = None
    r"""Content delta"""


DeltaTypedDict = TypeAliasType(
    "DeltaTypedDict", Union[ResponseStreamEvent2TypedDict, str]
)
r"""Delta updates (present in response.text.delta events), can be a string or object"""


Delta = TypeAliasType("Delta", Union[ResponseStreamEvent2, str])
r"""Delta updates (present in response.text.delta events), can be a string or object"""


class ResponseStreamEventTypedDict(TypedDict):
    r"""Response stream event (data payload for SSE stream)"""

    type: str
    r"""Event type (e.g., response.created, response.output_item.added, response.text.delta, response.completed)"""
    sequence_number: NotRequired[int]
    r"""Sequence number of the event"""
    response: NotRequired[ResponseObjectTypedDict]
    output_item: NotRequired[ResponseOutputItemTypedDict]
    output_item_index: NotRequired[int]
    r"""Index of the output item"""
    content_index: NotRequired[int]
    r"""Index of the content block"""
    delta: NotRequired[DeltaTypedDict]
    r"""Delta updates (present in response.text.delta events), can be a string or object"""


class ResponseStreamEvent(BaseModel):
    r"""Response stream event (data payload for SSE stream)"""

    type: str
    r"""Event type (e.g., response.created, response.output_item.added, response.text.delta, response.completed)"""

    sequence_number: Optional[int] = None
    r"""Sequence number of the event"""

    response: Optional[ResponseObject] = None

    output_item: Optional[ResponseOutputItem] = None

    output_item_index: Optional[int] = None
    r"""Index of the output item"""

    content_index: Optional[int] = None
    r"""Index of the content block"""

    delta: Optional[Delta] = None
    r"""Delta updates (present in response.text.delta events), can be a string or object"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .imageobject import ImageObject, ImageObjectTypedDict
from r9s.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ImageUsageTypedDict(TypedDict):
    """Usage information for image generation (provider-dependent)."""
    prompt_tokens: NotRequired[int]
    image_tokens: NotRequired[int]
    input_text_tokens: NotRequired[int]
    output_image_tokens: NotRequired[int]
    width: NotRequired[int]
    height: NotRequired[int]
    image_count: NotRequired[int]


class ImageUsage(BaseModel):
    """Usage information for image generation (provider-dependent)."""

    prompt_tokens: Optional[int] = None
    r"""Token count for the prompt."""

    image_tokens: Optional[int] = None
    r"""Token count for the generated image (GPT-Image-1/1.5)."""

    input_text_tokens: Optional[int] = None
    r"""Input text token count (Qwen)."""

    output_image_tokens: Optional[int] = None
    r"""Output image token count (Qwen)."""

    width: Optional[int] = None
    r"""Generated image width (Qwen)."""

    height: Optional[int] = None
    r"""Generated image height (Qwen)."""

    image_count: Optional[int] = None
    r"""Number of images generated (Qwen)."""


class ImageGenerationResponseTypedDict(TypedDict):
    created: int
    data: List[ImageObjectTypedDict]
    usage: NotRequired[ImageUsageTypedDict]


class ImageGenerationResponse(BaseModel):
    created: int

    data: List[ImageObject]

    usage: Optional[ImageUsage] = None
    r"""Usage information for image generation (provider-dependent)."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from pydantic.functional_validators import AfterValidator
from r9s.types import BaseModel
from r9s.utils import validate_const
from typing import Literal
from typing_extensions import Annotated, TypedDict


AnthropicDocumentSourceBase64MediaType = Literal[
    "application/pdf",
    "text/plain",
]
r"""Document MIME type"""


class AnthropicDocumentSourceBase64TypedDict(TypedDict):
    media_type: AnthropicDocumentSourceBase64MediaType
    r"""Document MIME type"""
    data: str
    r"""Base64-encoded document data"""
    type: Literal["base64"]


class AnthropicDocumentSourceBase64(BaseModel):
    media_type: AnthropicDocumentSourceBase64MediaType
    r"""Document MIME type"""

    data: str
    r"""Base64-encoded document data"""

    TYPE: Annotated[
        Annotated[Literal["base64"], AfterValidator(validate_const("base64"))],
        pydantic.Field(alias="type"),
    ] = "base64"



from __future__ import annotations

import json
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional

try:
    import tomllib  # pyright: ignore[reportMissingImports]
except Exception:  # pragma: no cover
    tomllib = None  # type: ignore[assignment]

try:
    import tomli  # pyright: ignore[reportMissingImports]
except Exception:  # pragma: no cover
    tomli = None  # type: ignore[assignment]


@dataclass
class CommandConfig:
    name: str
    description: Optional[str] = None
    prompt: Optional[str] = None


def commands_root() -> Path:
    return Path.home() / ".r9s" / "commands"


def command_path(name: str) -> Path:
    safe = name.strip()
    if not safe:
        raise ValueError("command name cannot be empty")
    return commands_root() / f"{safe}.toml"


def _load_toml(path: Path) -> Dict[str, Any]:
    raw = path.read_bytes()
    if tomllib is not None:
        data = tomllib.loads(raw.decode("utf-8"))
    elif tomli is not None:
        data = tomli.loads(raw.decode("utf-8"))
    else:
        raise RuntimeError("TOML parser is not available (need tomllib or tomli)")
    if not isinstance(data, dict):
        raise ValueError(f"invalid command config: {path}")
    return data


def _toml_quote(value: str) -> str:
    return json.dumps(value, ensure_ascii=False)


def _toml_multiline(value: str) -> str:
    if "'''" not in value:
        return "'''\n" + value + "\n'''"
    if '"""' not in value:
        return '"""\n' + value + '\n"""'
    return _toml_quote(value)


def _dump_command_toml(cmd: CommandConfig) -> str:
    lines: List[str] = []
    if cmd.description:
        lines.append(f"description = {_toml_quote(cmd.description)}")
    if cmd.prompt is not None:
        lines.append(f"prompt = {_toml_multiline(cmd.prompt)}")
    return "\n".join(lines).rstrip() + "\n"


def save_command(cmd: CommandConfig) -> Path:
    root = commands_root()
    root.mkdir(parents=True, exist_ok=True)
    path = command_path(cmd.name)
    path.write_text(_dump_command_toml(cmd), encoding="utf-8")
    return path


def load_command(name: str) -> CommandConfig:
    path = command_path(name)
    data = _load_toml(path)

    description_raw = data.get("description")
    description = (
        description_raw.strip()
        if isinstance(description_raw, str) and description_raw.strip()
        else None
    )

    prompt_raw = data.get("prompt")
    prompt = prompt_raw if isinstance(prompt_raw, str) else None
    prompt = prompt.strip() if (prompt and prompt.strip()) else None

    if not prompt:
        raise ValueError(f"command config missing 'prompt': {path}")

    return CommandConfig(name=name.strip(), description=description, prompt=prompt)


def list_commands() -> List[str]:
    root = commands_root()
    if not root.exists():
        return []
    return [p.stem for p in sorted(root.glob("*.toml"))]


def delete_command(name: str) -> Path:
    path = command_path(name)
    path.unlink()
    return path



from __future__ import annotations

from pathlib import Path

import pytest

from r9s.skills.exceptions import SecurityError, SkillNotFoundError
from r9s.skills.local_store import (
    delete_skill,
    list_skills,
    load_skill,
    save_skill,
    skill_path,
)
from r9s.skills.models import ScriptPolicy


def _skill_content() -> str:
    return """---
name: code-review
description: Review code
---

# Code Review

Use this skill.
"""


def test_skill_store_crud(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.setenv("R9S_SKILLS_DIR", str(tmp_path))
    save_skill("code-review", _skill_content())

    skill_dir = skill_path("code-review")
    (skill_dir / "scripts").mkdir(parents=True)
    (skill_dir / "references").mkdir(parents=True)
    (skill_dir / "assets").mkdir(parents=True)
    (skill_dir / "scripts" / "lint.sh").write_text("echo ok", encoding="utf-8")
    (skill_dir / "references" / "readme.md").write_text("doc", encoding="utf-8")
    (skill_dir / "assets" / "logo.png").write_bytes(b"img")

    assert list_skills() == ["code-review"]

    skill = load_skill("code-review")
    assert skill.name == "code-review"
    assert "scripts/lint.sh" in skill.scripts
    assert "references/readme.md" in skill.references
    assert "assets/logo.png" in skill.assets

    delete_skill("code-review")
    assert not skill_dir.exists()


def test_load_skill_not_found(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.setenv("R9S_SKILLS_DIR", str(tmp_path))
    with pytest.raises(SkillNotFoundError, match="not-a-skill"):
        load_skill("not-a-skill")


def test_delete_skill_not_found(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.setenv("R9S_SKILLS_DIR", str(tmp_path))
    with pytest.raises(SkillNotFoundError, match="missing-skill"):
        delete_skill("missing-skill")


def test_load_skill_with_scripts_blocked(tmp_path: Path, monkeypatch) -> None:
    monkeypatch.setenv("R9S_SKILLS_DIR", str(tmp_path))
    save_skill("script-skill", _skill_content().replace("code-review", "script-skill"))

    skill_dir = skill_path("script-skill")
    scripts_dir = skill_dir / "scripts"
    scripts_dir.mkdir()
    (scripts_dir / "run.sh").write_text("echo hi", encoding="utf-8")

    # Should fail with scripts blocked
    with pytest.raises(SecurityError, match="--allow-scripts"):
        load_skill("script-skill", policy=ScriptPolicy(allow_scripts=False))

    # Should pass with scripts allowed
    skill = load_skill("script-skill", policy=ScriptPolicy(allow_scripts=True))
    assert skill.name == "script-skill"
    assert "scripts/run.sh" in skill.scripts



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcompletionchoice import ChatCompletionChoice, ChatCompletionChoiceTypedDict
from .usage import Usage, UsageTypedDict
import pydantic
from pydantic.functional_validators import AfterValidator
from r9s.types import BaseModel
from r9s.utils import validate_const
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChatCompletionResponseTypedDict(TypedDict):
    id: str
    created: int
    model: str
    choices: List[ChatCompletionChoiceTypedDict]
    object: Literal["chat.completion"]
    usage: NotRequired[UsageTypedDict]
    system_fingerprint: NotRequired[str]


class ChatCompletionResponse(BaseModel):
    id: str

    created: int

    model: str

    choices: List[ChatCompletionChoice]

    OBJECT: Annotated[
        Annotated[
            Literal["chat.completion"],
            AfterValidator(validate_const("chat.completion")),
        ],
        pydantic.Field(alias="object"),
    ] = "chat.completion"

    usage: Optional[Usage] = None

    system_fingerprint: Optional[str] = None



"""Tests for image constraints utility module."""

from __future__ import annotations

from r9s.utils.image_constraints import (
    MODEL_CONSTRAINTS,
    GEMINI_ASPECT_RATIOS,
    MINIMAX_ASPECT_RATIOS,
    OPENAI_SIZE_RATIOS,
    ModelConstraints,
    get_gemini_aspect_ratio,
    get_minimax_dimensions,
    get_model_constraints,
    validate_image_request,
)


class TestModelConstraints:
    """Tests for ModelConstraints dataclass."""

    def test_model_constraints_creation(self) -> None:
        """ModelConstraints can be created with all fields."""
        constraints = ModelConstraints(
            sizes=["1024x1024", "512x512"],
            n_range=(1, 10),
            prompt_max=1000,
            supports_negative_prompt=True,
            supports_seed=True,
            supports_aspect_ratio=False,
            supports_4k=False,
        )
        assert constraints.sizes == ["1024x1024", "512x512"]
        assert constraints.n_range == (1, 10)
        assert constraints.prompt_max == 1000
        assert constraints.supports_negative_prompt is True
        assert constraints.supports_seed is True
        assert constraints.supports_aspect_ratio is False
        assert constraints.supports_4k is False

    def test_model_constraints_defaults(self) -> None:
        """ModelConstraints has correct default values."""
        constraints = ModelConstraints(
            sizes=["1024x1024"],
            n_range=(1, 1),
            prompt_max=1000,
        )
        assert constraints.supports_negative_prompt is False
        assert constraints.supports_seed is False
        assert constraints.supports_aspect_ratio is False
        assert constraints.supports_4k is False


class TestGetModelConstraints:
    """Tests for get_model_constraints function."""

    def test_get_dalle2_constraints(self) -> None:
        """Get constraints for DALL-E 2."""
        constraints = get_model_constraints("dall-e-2")
        assert constraints is not None
        assert "256x256" in constraints.sizes
        assert "512x512" in constraints.sizes
        assert "1024x1024" in constraints.sizes
        assert constraints.n_range == (1, 10)
        assert constraints.prompt_max == 1000

    def test_get_dalle3_constraints(self) -> None:
        """Get constraints for DALL-E 3."""
        constraints = get_model_constraints("dall-e-3")
        assert constraints is not None
        assert "1024x1024" in constraints.sizes
        assert "1024x1792" in constraints.sizes
        assert "1792x1024" in constraints.sizes
        assert constraints.n_range == (1, 1)
        assert constraints.prompt_max == 4000

    def test_get_gpt_image_1_constraints(self) -> None:
        """Get constraints for gpt-image-1."""
        constraints = get_model_constraints("gpt-image-1")
        assert constraints is not None
        assert constraints.n_range == (1, 1)
        assert constraints.prompt_max == 4000

    def test_get_gpt_image_15_constraints(self) -> None:
        """Get constraints for gpt-image-1.5."""
        constraints = get_model_constraints("gpt-image-1.5")
        assert constraints is not None
        assert constraints.n_range == (1, 10)  # Supports multiple images
        assert constraints.prompt_max == 4000

    def test_get_wanx_constraints(self) -> None:
        """Get constraints for wanx-v1."""
        constraints = get_model_constraints("wanx-v1")
        assert constraints is not None
        assert constraints.prompt_max == 800
        assert constraints.supports_negative_prompt is True
        assert constraints.supports_seed is True

    def test_get_gemini_constraints(self) -> None:
        """Get constraints for Gemini Nano Banana."""
        constraints = get_model_constraints("gemini-2.5-flash-image")
        assert constraints is not None
        assert constraints.prompt_max == 32000
        assert constraints.supports_aspect_ratio is True
        assert "1:1" in constraints.sizes
        assert "16:9" in constraints.sizes

    def test_get_gemini_pro_constraints(self) -> None:
        """Get constraints for Gemini Nano Banana Pro with 4K support."""
        constraints = get_model_constraints("gemini-3-pro-image-preview")
        assert constraints is not None
        assert constraints.supports_4k is True
        assert constraints.supports_aspect_ratio is True

    def test_get_cogview_constraints(self) -> None:
        """Get constraints for CogView."""
        constraints = get_model_constraints("cogview-3")
        assert constraints is not None
        assert constraints.prompt_max == 833
        assert constraints.n_range == (1, 1)

    def test_get_minimax_constraints(self) -> None:
        """Get constraints for Minimax image-01."""
        constraints = get_model_constraints("image-01")
        assert constraints is not None
        assert constraints.n_range == (1, 9)
        assert constraints.prompt_max == 1500
        assert constraints.supports_aspect_ratio is True
        assert constraints.supports_seed is True

    def test_get_unknown_model_returns_none(self) -> None:
        """Unknown model returns None."""
        constraints = get_model_constraints("unknown-model")
        assert constraints is None


class TestValidateImageRequest:
    """Tests for validate_image_request function."""

    def test_valid_request(self) -> None:
        """Valid request returns no errors."""
        errors = validate_image_request(
            model="dall-e-2",
            prompt="A beautiful sunset",
            size="1024x1024",
            n=1,
        )
        assert errors == []

    def test_prompt_too_long(self) -> None:
        """Prompt exceeding max length returns error."""
        long_prompt = "x" * 1001  # DALL-E 2 max is 1000
        errors = validate_image_request(
            model="dall-e-2",
            prompt=long_prompt,
        )
        assert len(errors) == 1
        assert "Prompt too long" in errors[0]
        assert "1001 > 1000" in errors[0]

    def test_invalid_size(self) -> None:
        """Invalid size returns error."""
        errors = validate_image_request(
            model="dall-e-2",
            prompt="Test",
            size="2048x2048",  # Not valid for DALL-E 2
        )
        assert len(errors) == 1
        assert "Invalid size" in errors[0]

    def test_invalid_n_below_range(self) -> None:
        """n below valid range returns error."""
        errors = validate_image_request(
            model="dall-e-2",
            prompt="Test",
            n=0,
        )
        assert len(errors) == 1
        assert "Invalid n" in errors[0]

    def test_invalid_n_above_range(self) -> None:
        """n above valid range returns error."""
        errors = validate_image_request(
            model="dall-e-3",
            prompt="Test",
            n=5,  # DALL-E 3 only supports n=1
        )
        assert len(errors) == 1
        assert "Invalid n" in errors[0]
        assert "Range: 1-1" in errors[0]

    def test_gpt_image_15_supports_multiple_n(self) -> None:
        """gpt-image-1.5 supports n up to 10."""
        errors = validate_image_request(
            model="gpt-image-1.5",
            prompt="Test",
            n=5,
        )
        assert errors == []

        errors = validate_image_request(
            model="gpt-image-1.5",
            prompt="Test",
            n=11,  # Above max
        )
        assert len(errors) == 1
        assert "Invalid n" in errors[0]

    def test_negative_prompt_unsupported(self) -> None:
        """negative_prompt on unsupported model returns error."""
        errors = validate_image_request(
            model="dall-e-3",
            prompt="Test",
            negative_prompt="blur, noise",
        )
        assert len(errors) == 1
        assert "does not support negative_prompt" in errors[0]

    def test_negative_prompt_supported(self) -> None:
        """negative_prompt on supported model returns no error."""
        errors = validate_image_request(
            model="wanx-v1",
            prompt="Test",
            negative_prompt="blur, noise",
        )
        assert errors == []

    def test_unknown_model_skips_validation(self) -> None:
        """Unknown model skips validation."""
        errors = validate_image_request(
            model="unknown-model",
            prompt="x" * 10000,  # Very long prompt
            size="9999x9999",
            n=100,
        )
        assert errors == []

    def test_multiple_errors(self) -> None:
        """Multiple validation errors are all returned."""
        errors = validate_image_request(
            model="dall-e-2",
            prompt="x" * 1001,  # Too long
            size="2048x2048",  # Invalid
            n=20,  # Above max
        )
        assert len(errors) == 3

    def test_gemini_aspect_ratio_valid(self) -> None:
        """Gemini accepts aspect ratio as size."""
        errors = validate_image_request(
            model="gemini-2.5-flash-image",
            prompt="Test",
            size="16:9",
        )
        assert errors == []

    def test_qwen_prompt_max(self) -> None:
        """Qwen has 800 character limit."""
        errors = validate_image_request(
            model="wanx-v1",
            prompt="x" * 801,
        )
        assert len(errors) == 1
        assert "801 > 800" in errors[0]


class TestGeminiAspectRatioMapping:
    """Tests for Gemini aspect ratio mapping."""

    def test_get_gemini_aspect_ratio_from_size(self) -> None:
        """Get aspect ratio from OpenAI-style size."""
        assert get_gemini_aspect_ratio("1024x1024") == "1:1"
        assert get_gemini_aspect_ratio("1792x1024") == "16:9"
        assert get_gemini_aspect_ratio("1024x1792") == "9:16"

    def test_get_gemini_aspect_ratio_passthrough(self) -> None:
        """Aspect ratio strings pass through unchanged."""
        assert get_gemini_aspect_ratio("1:1") == "1:1"
        assert get_gemini_aspect_ratio("16:9") == "16:9"
        assert get_gemini_aspect_ratio("21:9") == "21:9"

    def test_get_gemini_aspect_ratio_unknown(self) -> None:
        """Unknown size returns None."""
        assert get_gemini_aspect_ratio("9999x9999") is None

    def test_gemini_aspect_ratios_mapping_complete(self) -> None:
        """All expected mappings exist."""
        expected = {
            "1024x1024": "1:1",
            "1792x1024": "16:9",
            "1024x1792": "9:16",
            "1152x864": "4:3",
            "864x1152": "3:4",
            "1248x832": "3:2",
            "832x1248": "2:3",
            "1280x1024": "5:4",
            "1024x1280": "4:5",
            "1344x576": "21:9",
        }
        for size, ratio in expected.items():
            assert GEMINI_ASPECT_RATIOS.get(size) == ratio


class TestMinimaxDimensionsMapping:
    """Tests for Minimax dimensions mapping."""

    def test_get_minimax_dimensions(self) -> None:
        """Get pixel dimensions from aspect ratio."""
        assert get_minimax_dimensions("1:1") == (1024, 1024)
        assert get_minimax_dimensions("16:9") == (1280, 720)
        assert get_minimax_dimensions("9:16") == (720, 1280)

    def test_get_minimax_dimensions_unknown(self) -> None:
        """Unknown aspect ratio returns None."""
        assert get_minimax_dimensions("99:1") is None

    def test_minimax_aspect_ratios_mapping_complete(self) -> None:
        """All expected mappings exist."""
        expected = {
            "1:1": (1024, 1024),
            "16:9": (1280, 720),
            "4:3": (1152, 864),
            "3:2": (1248, 832),
            "2:3": (832, 1248),
            "3:4": (864, 1152),
            "9:16": (720, 1280),
            "21:9": (1344, 576),
        }
        for ratio, dims in expected.items():
            assert MINIMAX_ASPECT_RATIOS.get(ratio) == dims


class TestOpenAISizeRatios:
    """Tests for OpenAI size cost ratios."""

    def test_dalle2_ratios(self) -> None:
        """DALL-E 2 size ratios are correct."""
        assert OPENAI_SIZE_RATIOS["dall-e-2"]["256x256"] == 1.0
        assert OPENAI_SIZE_RATIOS["dall-e-2"]["512x512"] == 1.125
        assert OPENAI_SIZE_RATIOS["dall-e-2"]["1024x1024"] == 1.25

    def test_dalle3_ratios(self) -> None:
        """DALL-E 3 size ratios are correct."""
        assert OPENAI_SIZE_RATIOS["dall-e-3"]["1024x1024"] == 1.0
        assert OPENAI_SIZE_RATIOS["dall-e-3"]["1024x1792"] == 2.0
        assert OPENAI_SIZE_RATIOS["dall-e-3"]["1792x1024"] == 2.0

    def test_gpt_image_ratios(self) -> None:
        """GPT-Image size ratios are correct."""
        for model in ["gpt-image-1", "gpt-image-1.5"]:
            assert OPENAI_SIZE_RATIOS[model]["1024x1024"] == 1.0
            assert OPENAI_SIZE_RATIOS[model]["1024x1536"] == 2.0
            assert OPENAI_SIZE_RATIOS[model]["1536x1024"] == 2.0


class TestModelConstraintsDict:
    """Tests for MODEL_CONSTRAINTS dictionary."""

    def test_all_openai_models_present(self) -> None:
        """All OpenAI models are in constraints dict."""
        openai_models = ["dall-e-2", "dall-e-3", "gpt-image-1", "gpt-image-1.5"]
        for model in openai_models:
            assert model in MODEL_CONSTRAINTS

    def test_all_gemini_models_present(self) -> None:
        """All Gemini models are in constraints dict."""
        gemini_models = [
            "gemini-2.5-flash-image",
            "gemini-2.5-flash-preview-05-20",
            "gemini-3-pro-image-preview",
        ]
        for model in gemini_models:
            assert model in MODEL_CONSTRAINTS

    def test_all_ali_models_present(self) -> None:
        """All Ali/Qwen models are in constraints dict."""
        ali_models = ["wanx-v1", "ali-stable-diffusion-xl", "ali-stable-diffusion-v1.5"]
        for model in ali_models:
            assert model in MODEL_CONSTRAINTS

    def test_other_models_present(self) -> None:
        """Other provider models are in constraints dict."""
        other_models = ["cogview-3", "image-01"]
        for model in other_models:
            assert model in MODEL_CONSTRAINTS



"""Lazy-loaded rich markdown rendering for chat output.

Rich is an optional dependency. If not installed, falls back to plain text.
Install with: pip install r9s[rich]
"""

from __future__ import annotations

import os
from typing import Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from rich.console import Console

# Lazy-loaded rich module
_rich_console: Optional["Console"] = None
_rich_available: Optional[bool] = None


def is_rich_enabled() -> bool:
    """Check if rich output is enabled via --rich flag or R9S_RICH env."""
    return os.getenv("R9S_RICH", "").lower() in ("1", "true", "yes")


def is_rich_available() -> bool:
    """Check if rich library is installed (lazy check)."""
    global _rich_available
    if _rich_available is None:
        try:
            import rich  # noqa: F401
            _rich_available = True
        except ImportError:
            _rich_available = False
    return _rich_available


def _get_console() -> "Console":
    """Get or create rich Console instance (lazy)."""
    global _rich_console
    if _rich_console is None:
        from rich.console import Console
        _rich_console = Console()
    return _rich_console


def print_markdown(text: str) -> None:
    """Print text as markdown using rich, or plain text if unavailable.

    Args:
        text: The markdown text to render
    """
    if not is_rich_available():
        print(text)
        return

    from rich.markdown import Markdown
    console = _get_console()
    md = Markdown(text)
    console.print(md)


def print_response(text: str, *, use_rich: bool = False) -> None:
    """Print assistant response, optionally with rich markdown rendering.

    Args:
        text: The response text
        use_rich: Whether to use rich markdown rendering
    """
    if use_rich and is_rich_available():
        print_markdown(text)
    else:
        print(text)


def format_tokens(input_tokens: int, output_tokens: int) -> str:
    """Format token usage for display.

    Args:
        input_tokens: Number of input/prompt tokens
        output_tokens: Number of output/completion tokens

    Returns:
        Formatted string like "tokens: 150 in / 89 out"
    """
    return f"tokens: {input_tokens} in / {output_tokens} out"
